PROGRAM DMFTMAIN
! Program DMFT calculates 
  USE param
  USE struct
  USE case
  USE sym2
  USE com
  USE abc
  USE kpts, only : numkpt, MKNAME, MIPGR, mweight, tweight, MSX, MSY, MSZ, allocate_kpts, deallocate_kpts
  USE com_mpi
  IMPLICIT REAL*8 (A-H,O-Z)
  complex*16, allocatable :: trmat(:,:)
  real*8, allocatable     :: rcf(:,:)
  complex*16      qtl1(24), IMAG
  dimension       xx(3),xz(3),acf(14),bcf(14),qtle(34)
  CHARACTER*4     adum
  CHARACTER*5     MODSYM,CHAR
  CHARACTER*10    KNAME
  CHARACTER*11    STATUS,FORM                                      
  CHARACTER*67    ERRMSG
  CHARACTER*80    DEFFN, ERRFN
  CHARACTER*200   FNAME, enefn,enefnd, fUdmft
  CHARACTER*200   VECFN
  CHARACTER*161   TEXT
  INTEGER,ALLOCATABLE :: ieband(:)        
  INTEGER         IPROC
  LOGICAL         SO, Qcomplex, newform, Qident, Qrenormalize
  CHARACTER*1  :: mode
  INTEGER         kindex, nsymop
  INTEGER         wndim, icix, wicix, size, imatsubara, irenormalize
  REAL*8       :: EF_LDA, Ry2eV
  REAL*8,ALLOCATABLE:: wtmp(:)
  INTEGER      :: projector
  INTEGER      :: strlen, locrot, shift, latom, natm, wfirst, iat, im, wat, minsigind
  COMMON /GENER/  BR1(3,3),BR2(3,3)                                 
  COMMON /RADFU/  RRAD1(NRAD,0:LMAX2),RADE1(NRAD,0:LMAX2), RRAD2(NRAD,0:LMAX2),RADE2(NRAD,0:LMAX2)
  COMMON /PROC/    VECFN(2)
  COMMON /IPROC/   IPROC 
  DIMENSION  S(3)
  DATA SO /.false./, Qcomplex /.false./, IMAG/(0.0D0,1.0D0)/, Ry2eV/13.60569193/
  DATA strlen/200/
  !------------------------------------------------------------------     
  
  fUdmft = 'Udmft.'

  CALL start_MPI()
  
  CALL GTFNAM(DEFFN,ERRFN,IPROC)
  
  CALL ERRFLG(ERRFN,'Error in DMFT')

  nspin1=1
  OPEN (1,FILE=DEFFN,STATUS='OLD',ERR=910,IOSTAT=ios) ! opens dmftx.def file
  
  iso=1
  iso2=1
  ! The following few lines read dmft0.def file containing all filename definitions
  !
  DO 
     READ (1,*,END=20,ERR=960) IUNIT,FNAME,STATUS,FORM,IRECL  ! can jump to 20
     !print *, 'Opening file', FNAME
     if(iunit.eq.6) CALL FilenameMPI(FNAME)    ! Each processor will have different information file 

     if (iunit.lt.100  .or. myrank.eq.master) then            ! output files (iunit>=100) should be open on master node only
        OPEN (IUNIT,FILE=FNAME,STATUS=STATUS,FORM=FORM,ERR=920)
     endif

     if(iunit.eq.9)  VECFN(1)=FNAME
     if(iunit.eq.10) VECFN(2)=FNAME
     if(iunit.eq.18) then
        do i=strlen,5,-1
           if(fname(i:i).ne.' ') then
              if(fname(i-2:i).eq.'pup') nspin1=2
              if(fname(i-2:i).eq.'pdn') nspin1=2  ! added 10.07.08 PN
           endif
        enddo
     endif
     if(iunit==59) enefnd=fname
     if(iunit==60) enefn=fname
     if(iunit.eq.7) then
        read(iunit,123,end=12)CHAR
        Qcomplex=.true. ! switches Qcomplex  on if case.in1c exists and is nonzero
12      continue
     end if
     if(iunit.eq.399) then
        do i=strlen,2,-1
           if(fname(i:i).ne.' ') then
              fUdmft = fname(:i-1)
              exit
           endif
        enddo
        if (myrank.eq.master) close(399) ! Will reopen later
        !print *, 'Found 399=', fUdmft
     endif
     if(iunit.eq.9) then
        iloop1: do i=strlen,2,-1
           if(fname(i:i).ne.' ') then
              if(fname(i-1:i).eq.'so') then
                 SO=.true.
                 iso=2
                 iso2=1
                 Qcomplex=.true.
                 exit iloop1
              endif
           endif
        enddo iloop1
        iloop2: do i=strlen,4,-1
           if(fname(i:i).ne.' ') then
              if(fname(i-3:i).eq.'soup') then
                 SO=.true.
                 iso=2
                 iso2=2
                 Qcomplex=.true.
                 exit iloop2
              endif
           endif
        enddo iloop2
     endif
  ENDDO ! 10
20 CONTINUE
  CLOSE (1)
  ! finished dtft0.def

  !.....READING STRUCT 
  ! It reads case.struct file and saves data into module named "param". These variables are available in dmftmain.f.
  CALL init_struct                                                      
  write(6,'(A)')'******Program DMFT: Computes Gloc and Delta for Dynamical Mean Field Theory calculation ******'
  if (Qcomplex) then
     write(6,*)'CALCULATION WITH COMPLEX VECTORS'
  endif
  if (so)then
     if(iso2.eq.2)write(6,*)'CALCULATION WITH SOC-UP AND DN VECTORS REQUIRED'
     if(iso2.eq.1)write(6,*)'CALCULATION WITH SOC VECTORS REQUIRED'
  endif
  IF(IPROC.GT.0)THEN
     write(6,*) 'Running DMFT on ', IPROC, ' processors'
     write(6,*) ' '
     DO is=1,2
        call mknam(FNAME,VECFN(is),1)
        iunit=8+is
        status='UNKNOWN'
        form='UNFORMATTED'
        OPEN (IUNIT,FILE=FNAME,STATUS=STATUS,FORM=FORM,ERR=920)
     END DO
  ELSE
     write(6,*)'Running DMFT in single processor mode'
     write(6,*)' '
  END IF

  
  ! find Fermi energy using scf2 file
  EF = 0.0
  EF_LDA =0.0
  DO 
     read(8,666,end=14) adum
     if(adum.eq.':NOE') then
        read(8,*)
        read(8,667) EF_LDA
        exit
     endif
  ENDDO
  EF = EF_LDA
14 continue  ! Fermi energy end


  ! read klist and associated weights from file 66==case.klist

  ! first read how many k-points exists and the old/new file type
  ITAPE = 14
  numkpt = 0
  newform = .TRUE.
  DO
     READ (ITAPE, '(A20)', IOSTAT=ios) TEXT
     IF (numkpt==0 .AND. TEXT(15:16) .NE. ' ') THEN
        newform=.FALSE.
     ENDIF
     KNAME = TEXT(1:10)
     IF (KNAME .EQ. 'END       ' .OR. ios.ne.0) EXIT
     numkpt = numkpt + 1
  ENDDO
  REWIND(ITAPE)

  WRITE(6,*) 'numkpt=', numkpt
  
  CALL allocate_kpts(numkpt)
  MWEIGHT=0
  DO kindex=1,numkpt
     IF(newform) THEN
        READ (ITAPE,5101) MKNAME(KINDEX), ISX, ISY, ISZ, IDV, MWEIGHT(KINDEX), E1, E2, MIPGR(KINDEX)
     ELSE
        READ (ITAPE,5100) MKNAME(KINDEX), ISX, ISY, ISZ, IDV, MWEIGHT(KINDEX), E1, E2, MIPGR(KINDEX)
     ENDIF
     MSX(KINDEX) = DBLE(ISX)/DBLE(IDV)
     MSY(KINDEX) = DBLE(ISY)/DBLE(IDV)
     MSZ(KINDEX) = DBLE(ISZ)/DBLE(IDV)
     WRITE(6,'(4f10.5,1x)') MSX(kindex), MSY(kindex), MSZ(kindex), MWEIGHT(kindex)
  END DO
  TWEIGHT = sum(MWEIGHT)
  !print *, 'twgh=', myrank, tweight

  ! iord : number of symmetry operations from case.struct
  CALL init_sym2(iord) ! allocates memory for: idet(iord),opimat(3,3,iord),phase(iord),tmat(3,3,iord)
  WRITE(6,800) 
  WRITE(6,805)   TITLE                                               
  WRITE(6,810)   LATTIC                                              
  WRITE(6,820)   AA,BB,CC                                            
  WRITE(6,840)   NAT                                                 
  WRITE(6,850)   IREL                             
  !write(30,4030) title   ! Ordering of DOS in DMFT file for:
  write(6,*)'IORD=',IORD

!     LATGEN GENERATES TWO BRAVAIS MATRICES, DEFINES THE VOLUME OF      
!     THE UNIT CELL AND CALLS ROTDEF                                    
!     BR1(3,3)  : TRANSFORMS INTEGER RECIPROCAL LATTICE VECTORS AS      
!                 GIVEN IN THE VECTORLIST OF LAPW1  ( GENERATED IN      
!                 COORS, TRANSFORMED IN BASISO, AND WRITTEN OUT IN      
!                 WFTAPE) INTO CARTESIAN SYSTEM                         
!     BR2(3,3) :  TRANSFORMS A RECIPROCAL LATTICE VECTOR OF A SPE-      
!                 CIAL COORDINATE SYSTEM ( IN UNITS OF 2 PI / A )       
!                 TO CARTESIAN SYSTEM                                   
!      
  CALL LATGEN ! for results look at case.outputq  


! Reads current input file case.indmf0
! 5 is connected to current input file case.indmf0
  !print *, 'Start reading 5'
  READ(5,*) EMIN,EMAX,irenormalize,projector
  EMIN = EMIN/Ry2eV
  EMAX = EMAX/Ry2eV
  !print *, 'Emin,Emax=', Emin, Emax
  READ(5,*) imatsubara, gammac, gamma, nom_default, aom_default, bom_default
  !print *, 'imatsubara=', imatsubara, gammac, gamma, Qmatrix, nom_default, aom_default, bom_default
  read(5,*) natom
  Qrenormalize = irenormalize
  matsubara = imatsubara

  write(6, 877) Emin,Emax  ! Energy window
  write(6, 887) natom      ! Projected density of states calculation for xxx atoms
  !write(21,877) Emin,Emax
  !write(21,887) natom
  
  ALLOCATE(nl(natom))
  allocate(ll(natom,4), qsplit(natom,4), cix(natom,4), iatom(natom))
  ALLOCATE(shft(natom,3), crotloc(3,3,natom))
  crotloc=0
  shft = 0

  natm = sum(mult)
  ALLOCATE( isort(natm), ifirst(natm) )
  wfirst  = 1          ! first atom of this particular sort
  do iat=1,nat         ! over all sorts
     do im=1,MULT(iat) ! over all atoms of this sort
        wat = wfirst + im-1  ! atom number
        isort(wat)=iat       ! sort of each atom
        ifirst(wat)=wfirst
     enddo
     wfirst = wfirst + MULT(iat)
  enddo
  !print *, 'isort=', isort


  !!!!!!!!!!!!!!!!!!!!!!!!!!!!

  DO i=1,natom
     READ(5,*) latom,nL(i),locrot ! read from case.inq
     
     iatom(i) = latom       ! The succesive number of atom (all atoms counted)
     jatom = isort(latom)   ! The sort of the atom

     loro = MOD(locrot, 3)
     shift = locrot/3
     
     WRITE(6,'(A,I2,A)'), '--------- atom ', i, ' -------------'

     ! loro can be (0 -original coordinate system), (1 - new z-axis), (2 - new z-axis, x-axis)
     do j=1,nL(i)
        read(5,*) LL(i,j), qsplit(i,j), cix(i,j) ! LL contains all L-quantum numbers which need to be computed.
     enddo

     do j=1,nL(i)
        write(6, '(A,I3,2x,A,I3)') 'l=', LL(i,j), 'qsplit=', qsplit(i,j)!, 'nrfmax=', nrfmax(i,j)
        !write(21,'(A,I3,2x,A,I3)') 'l=', LL(i,j), 'qsplit=', qsplit(i,j)!, 'nrfmax=', nrfmax(i,j)
     enddo
     write(6,*)'Symmetrization over eq. k-points is performed'
     
     crotloc(:,:,i) = rotloc(:,:,jatom)  ! Rotation for this type from the struct file

     !print *, 'i=', i, 'jatom=', jatom
     !print *, 'rotloc=', crotloc(:,:,i)

     ! Rotation matrix in case the coordinate system is changed!
     if(loro.gt.0)then        ! change of local rotation matrix
        read(5,*)(xz(j),j=1,3)  ! new z-axis expressed in unit cell vectors
        write(6,120)(xz(j),j=1,3)
        !write(21,120)(xz(j),j=1,3)
        call angle(xz,thetaz,phiz)
        crotloc(1,3,i)=sin(thetaz)*cos(phiz)
        crotloc(2,3,i)=sin(thetaz)*sin(phiz)
        crotloc(3,3,i)=cos(thetaz)          
        if(loro.eq.1)then       ! only z-axis fixed
           crotloc(3,1,i)= 0.     ! new x perpendicular to new z       
           ddd=abs(crotloc(3,3,i)**2-1.)
           if(ddd.gt.0.000001)then
              crotloc(1,1,i)=-crotloc(2,3,i)
              crotloc(2,1,i)= crotloc(1,3,i)
           else
              crotloc(1,1,i)=1.
              crotloc(2,1,i)=0.
           endif
           ddd=0.                 ! normalize new x
           do j=1,3
              ddd=ddd+crotloc(j,1,i)**2
           enddo
           do j=1,3
              crotloc(j,1,i)=crotloc(j,1,i)/sqrt(ddd)
           enddo
        elseif(loro.eq.2)then   ! also new x-axis fixed
           read(5,*)(xx(j),j=1,3)
           write(6,121)(xx(j),j=1,3)
           !write(21,121)(xx(j),j=1,3)
           call angle(xx,thetax,phix)
           crotloc(1,1,i)=sin(thetax)*cos(phix)
           crotloc(2,1,i)=sin(thetax)*sin(phix)
           crotloc(3,1,i)=cos(thetax)          
           !  check orthogonality of new x and z axes
           check=0.
           do j=1,3
              check=check+crotloc(j,1,i)*crotloc(j,3,i)
           enddo
           if(abs(check).gt.0.00001)then
              write(6,*)' new x and z axes are not orthogonal'
              print *,' new x and z axes are not orthogonal'
              stop
           endif
        endif
        crotloc(1,2,i)=crotloc(2,3,i)*crotloc(3,1,i)-crotloc(3,3,i)*crotloc(2,1,i)
        crotloc(2,2,i)=crotloc(3,3,i)*crotloc(1,1,i)-crotloc(1,3,i)*crotloc(3,1,i)
        crotloc(3,2,i)=crotloc(1,3,i)*crotloc(2,1,i)-crotloc(2,3,i)*crotloc(1,1,i)
        write(6,*)' New local rotation matrix in global orthogonal system'
        write(6,*)'                      new x     new y     new z'
        !write(21,*)' New local rotation matrix in global orthogonal system'
        !write(21,*)'                      new x     new y     new z'
        write(6,1013)((crotloc(j,j1,i),j1=1,3),j=1,3)  !written as in .struct
        !write(21,1013)((crotloc(j,j1,i),j1=1,3),j=1,3)
     endif   !change of loc.rot. end
     ! end Rotation matrix loro

     if (shift.gt.0) then
        read(5,*)(shft(i,j),j=1,3)
        write(6,'(A,1x,I2,A,2x,I3,I3,I3)') '** atom', i, ' has nonzero shift: ', shft(i,1), shft(i,2), shft(i,3)
     endif

     write(6,'(A,I2,A)') 'The actual position of the atom ', i, ' is:'
     write(6,'(f10.6,2x,f10.6,2x,f10.6)') POS(1,latom)+shft(i,1), POS(2,latom)+shft(i,2), POS(3,latom)+shft(i,3)
     
  enddo

  READ(2,*) ! comment: mode and the current chemical potential 
  READ(2,*) mode, EF, nsymop
  EF = EF/Ry2eV

  READ(5,*) ! comment: Next few lines contain instructions (transformation,index) for all correlated orbitals
  READ(5,*) ncix, maxdim, maxsize ! number of independent cix blocks
  WRITE(6,*) '********** Start Reading Cix file *************'
  ALLOCATE(CF(maxdim,maxdim,ncix))      ! transformation matrix
  ALLOCATE(Sigind(maxdim,maxdim,ncix))  ! correlated index
  ALLOCATE(legend(maxsize, ncix))       ! names of correlated orbitals
  ALLOCATE(csize(ncix))                 ! independent components
  ALLOCATE(wtmp(2*maxdim))              ! temp
  
  CF=0
  Sigind=0  
  Qident=.True.
  do icix=1,ncix
     READ(5,*) wicix, wndim, size  ! icix, size-of-matrix, L, number of independent components of the matrix

     csize(icix)=size
     if (wicix.ne.icix) then
        print *, 'Something wrong reading case.indmf0 file. Boilig out...'
        goto 999
     endif
     READ(5,*) ! Comment: Independent components are
     READ(5,*) (legend(i,icix),i=1,size)

     READ(5,*) ! Comment: Sigind follows
     do i=1,wndim
        READ(5,*) (Sigind(i,j,icix),j=1,wndim)
     enddo


     ! modify Sigind so that indices always begin with 1
     ! Example:
     ! 3 0 0 0 0        1 0 0 0 0
     ! 0 3 0 0 0        0 1 0 0 0
     ! 0 0 4 0 0  --->  0 0 2 0 0
     ! 0 0 0 4 0        0 0 0 2 0
     ! 0 0 0 0 4        0 0 0 0 2
     minsigind=10000
     do i=1,wndim
        do j=1,wndim
           if (Sigind(i,j,icix).ne.0) then
              minsigind = min(minsigind,Sigind(i,j,icix))
           endif
        enddo
     enddo
     
     if (minsigind.ne.1) then
        do i=1,wndim
           do j=1,wndim
              if (Sigind(i,j,icix).ne.0) then
                 Sigind(i,j,icix) = Sigind(i,j,icix)-minsigind+1
              endif
           enddo
        enddo
        
        WRITE(6,*) 'Sigind corrected to'
        do m1=1,wndim
           do m2=1,wndim
              write(6,'(I3)',advance='no') Sigind(m1,m2,icix)
           enddo
           write(6,*)
        enddo
     endif

     READ(5,*) ! Comment: Transformation matrix follows
     do i=1,wndim
        READ(5,*) (wtmp(j),j=1,2*wndim)
        do j=1,wndim
           CF(i,j,icix) = dcmplx(wtmp(2*j-1),wtmp(2*j))
        enddo
     enddo

     do i=1,wndim
        if (.not.Qident) exit
        do j=1,wndim
           if (i.eq.j) then
              ident=1
           else 
              ident=0
           endif
           if (abs(CF(i,j,icix)-ident)>1e-5) then
              Qident=.False.
              exit
           endif
        enddo
     enddo

     write(6,*)' Correlated block number', icix
     write(6,*)' Correlated index='
     do m1=1,wndim
        do m2=1,wndim
           write(6,'(I3)',advance='no') Sigind(m1,m2,icix)
        enddo
        write(6,*)
     enddo
     write(6,*)' Real part of unitary matrix='
     do m1=1,wndim
        do m2=1,wndim
           write(6,'(F8.4)',advance='no') dble(cf(m1,m2,icix))
        enddo
        write(6,*)
     enddo
     write(6,*)' Imaginary part of unitary matrix='
     do m1=1,wndim
        do m2=1,wndim
           write(6,'(F8.4)',advance='no') aimag(cf(m1,m2,icix))
        enddo
        write(6,*)
     enddo
  enddo
  DEALLOCATE(wtmp)
  
  if (Qident) then
     write(6,*)' All transformation matrices are Identity'
  else
     write(6,*)' At least one transformation matrix is not Identity'
  endif

  ! For spin-orbit coupling crates transformation "rotloc"      
  !....reading *.inso
  if (so) then
     do i=1,3
        read(4,*)
     end do
     read(4,*)s(1),s(2),s(3)
     call angle(s,theta,fi)
     write(6,126)theta,fi
     !write(21,126)theta,fi
     !.... test of collinearity of the z-axis with spin quantization axis
     cost=cos(theta)
     cosf=cos(fi)
     sint=sin(theta)
     sinf=sin(fi)
     xz(1)=sint*cosf
     xz(2)=sint*sinf
     xz(3)=cost
     do i=1,natom
        dsum=crotloc(3,1,i)*xz(1)+crotloc(3,2,i)*xz(2)+crotloc(3,3,i)*xz(3)
        if(abs(dsum-1.).gt.0.00001)then  ! old z not along M, new rotloc
           crotloc(1,1,i)=cosf*cost
           crotloc(2,1,i)=-sinf
           crotloc(3,1,i)=cosf*sint
           crotloc(1,2,i)=sinf*cost
           crotloc(2,2,i)=cosf
           crotloc(3,2,i)=sinf*sint
           crotloc(1,3,i)=-sint
           crotloc(2,3,i)=0
           crotloc(3,3,i)=cost
           write(6,*)' New local rotation matrix with z || M'
           !write(21,*)' New local rotation matrix with z || M'
           write(6,1013)((crotloc(j,j1,i),j1=1,3),j=1,3)  ! written as in .struct
           !write(21,1013)((rotloc(j,j1,i),j1=1,3),j=1,3)
        endif
     end do
  end if  ! spin-orbit

  
  !---------------------------------------------------------------------------------!
  !--- WRITE Output for the BR1 matrix and the ROTIJ Matrix in file case.rotlm:  ---!
  !---  they are read by LectureROTIJ in lecture.f in SRC_elnes / or SRC_mdff/.  ---!
  !---  These lines are in SRC_lapw2lm/l2main.frc: surch *PH*                    ---!
  !---  The number of the output file is 22. his extention must be .rotlm        ---!
  !---------------------------------------------------------------------------------!
  CALL ROTDEF 
  if (myrank.EQ.master) then

     WRITE(22,*) 'BR1'
     DO JR=1, 3
        WRITE(22, 9101) BR1(1,JR), BR1(2,JR), BR1(3,JR) !--- Writting BR1 ----!
     ENDDO
     WRITE(22,*) 'BR2'
     DO JR=1, 3
        WRITE(22, 9101) BR2(1,JR), BR2(2,JR), BR2(3,JR) !--- Writting BR1 ----!
     ENDDO
 
     INDEX = 0
     DO LATOM=1, NAT
        DO LATEQ=1, MULT(LATOM)
           INDEX = INDEX + 1
           WRITE(22, 9100) LATOM, LATEQ, INDEX !---- inequivalent atomnumber  ---!
           DO JR=1, 3
              WRITE(22, 9101) (ROTIJ(JC,JR,INDEX),JC=1,3) !--- Writting ROTIJ ---!
           ENDDO
        ENDDO
     ENDDO
  endif



  ! If SO-coupling and symmetrization over all k-points
  if (so) then
     CALL SYM(THETA,FI)
  else
     do i=1,3
        do j=1,3
           iz(i,j,1)=0
           if (i.eq.j) iz(i,j,1)=1
        end do
     end do
  endif


  !*****  finds nkpt, nmat and nume in energy file *****
  ! 59 -> case.energydn
  ! 60 -> case.energy
  ! using case.energy, we will find out what is:
  !  nkpt -- number of all k-points
  !  nmat -- maximum number of reciprocal vectors
  !  nume -- maximum number of bands kept
  k=0
  nume=0
  iloop=0
  DO ! 778  continue
     if(IPROC.GT.0) then
        iloop=iloop+1
        close(59)
        close(60)
        call mknam(FNAME,enefn,ILOOP)
        open(60,FILE=FNAME,STATUS='old',ERR=951)
        call mknam(FNAME,enefnd,ILOOP)
        open(59,FILE=FNAME,STATUS='unknown',ERR=951)
     endif
     jspin=1       !pb
     DO I=1,NAT
        READ(60,'(f9.5)') EMIST
        READ(60,'(f9.5)') EMIST
     ENDDO
     DO I=1,NAT
        READ(59,'(f9.5)',END=1005) EMIST
        READ(59,'(f9.5)',END=1005) EMIST
     ENDDO
     JSPIN=2
1005 CONTINUE
     DO
        READ(60,'(3e19.12,a10,2i6)',IOSTAT=ios) SS,T,Z,KNAME,N,NEn
        IF (ios /= 0) EXIT
        k=k+1
        nmat=MAX(n,nmat)
        nume=MAX(nen,nume)
        DO ii=1,nen
           READ(60,*) NUM,E1
        ENDDO
        IF(jspin.EQ.2) THEN
           READ(59,'(3e19.12,a10,2i6)',IOSTAT=ios) SS,T,Z,KNAME,N,NEn
           nmat=MAX(n,nmat)
           nume=MAX(nen,nume)
           DO ii=1,nen
              READ(59,*) NUM,E1
           ENDDO
        ENDIF
     ENDDO
     IF (ILOOP.GE.IPROC) EXIT
  ENDDO ! IF(ILOOP.LT.IPROC) goto 778

  nkpt=k
  REWIND(59)
  REWIND(60)
  CALL init_com(nume,nkpt)  ! Allocates weigh(nkpt,nume) and nb(nkpt) in module com.
  CALL init_abc(nume,nmat,lmax2,ndim2,nrf) ! Allocates variables:
  !    kx(nmat), ky(nmat), kz(nmat), bkx(nmat), bky(nmat), bkz(nmat), a(nmat,nume,2), alm(2*lmax2+1,nume,nrf,2), alml(0:lmax2,2*lmax2+1,nume,nrf,2), e(nume), xden(0:lmax2,ndim2,nume), xqtl(0:lmax2,0:lmax2,ndim2,ndim2,nume), dmat(0:lmax2,0:lmax2,ndim2,ndim2)
  !    in module abc

  !!!!!!
  Cohfacts = .False.
  IF (mode.EQ.'p') THEN
     ! Check if we want to use coherence factors for plotting bands
     OPEN(1001,FILE='cohfactors.dat',STATUS='OLD',ERR=992)
     READ(1001,*,IOSTAT=ios) TEXT
     if (ios /= 0) goto 992
     Cohfacts = .True.
  ENDIF
992 CONTINUE
  !!!!!!
  IF (mode.EQ.'p') mode='e' ! for plotting, we just print eigenvalues

  ! If 'EF.dat' exists, we should use it instead of case.indm2
  OPEN(1000,FILE='EF.dat',STATUS='OLD',ERR=991)
  READ(1000,*) EF
  EF = EF/Ry2eV
  WRITE(6,*) 'Found EF.dat and EF is ', EF
991 CONTINUE
  WRITE(6,*) 'EF finally set to ', EF*Ry2eV
  

  WRITE(*,'(A,A1,3x,A,F10.7)') 'mode=', mode, 'EF=', EF*Ry2eV
  !nsymop = iord
  !if (mode.EQ.'e') then
  !   nsymop = 1
  !endif
  if (nsymop.EQ.0) nsymop = iord


  call l2main(Qcomplex,nsymop,Qident,mode,projector,Qrenormalize,fUdmft)  !!!! This calls the main subroutine

  DEALLOCATE(CF, Sigind, csize, legend)
  DEALLOCATE(shft)
  DEALLOCATE(isort, ifirst)
  DEALLOCATE(crotloc)
  CALL deallocate_kpts


  CALL CPUTIM(TTIME)
                                                                       
!.....CALCULATE CPUTIME REQUIRED                                        
  WRITE(6,2000)                                                     
  WRITE(6,2010) TTIME                                        
  CALL ERRCLR(ERRFN)

  CALL stop_MPI()

  STOP ' DMFT END'

  
!   error handling
!
910 INFO = 1
!
!   'DMFTx.def' couldn't be opened
!910
  print *, 'Definition file error', TRIM(DEFFN), 'could not be opened!'
  print *, 'ios=', ios
  WRITE (ERRMSG,9000) DEFFN
  CALL OUTERR('DMFT',ERRMSG)
  GOTO 999

920 INFO = 2
!
!  file FNAME couldn't be opened
!
  WRITE (ERRMSG,9010) IUNIT
  CALL OUTERR('DMFT',ERRMSG)
  WRITE (ERRMSG,9020) FNAME
  CALL OUTERR('DMFT',ERRMSG)
  WRITE (ERRMSG,9030) STATUS, FORM
  CALL OUTERR('DMFT',ERRMSG)
  GOTO 999
951 INFO = 5
  write(6,*) 'error:',FNAME
  CALL OUTERR('DMFT','file open error')
  GOTO 999
960 INFO = 7
!
!        Error reading file 'QTL.def'
!
  WRITE (ERRMSG,9040) FNAME
  CALL OUTERR('DMFT',ERRMSG)
  GOTO 999
999 STOP 'DMFT - Error'
!                                                                       
!                                                                       

9100 FORMAT('inequivalent atomnumber ',I3,' number ',I2,' total ',I4)
9101 FORMAT(3F10.5)

123 FORMAT(A5)
815 FORMAT(' Compound:',a60)
666 FORMAT(a4)
667 FORMAT(38x,f10.5)
887 FORMAT(' Projected density of states calculation for',i3,' atoms')
877 FORMAT(' Energy window:',f10.4,' < E < ',f10.3)
4030 FORMAT(' Ordering of DOS in DMFT file for: ',a60,/)
120 FORMAT(' New z axis || ',3f9.4)
121 FORMAT(' New x axis || ',3f9.4)
1013 FORMAT('LOCAL ROT MATRIX:   ',3f10.7,/,20x,3f10.7,/,20x,3f10.7)
101 FORMAT(' L=',i2,'. Unitary transFORMATion read from input')
555 FORMAT(7f9.4)
111 FORMAT(' Population matrix diagonal in L for L=',4i3)
110 FORMAT(' Complete population matrix with <L1|L2> terms for L=',4i3)
112 FORMAT(' Number of cases for TETRA=',i4)
776 FORMAT(4f12.6,' Emin,dE,Emax,dgauss')
777 FORMAT(i5,'                         number of cases for tetra')
126 FORMAT(' Magnetic system with s-o coupling; M theta, phi:',2f8.4) 
4031 FORMAT(' atom',i4,' ordering of projected DOS ')
987 FORMAT(a5)              
1052 FORMAT(' Data for interstital DOS correspond to atom index ',i4)
1050 FORMAT(F10.5,I3,I5,F8.5,3X,39F8.5)
1051 FORMAT(F10.5,I3,F8.5,3X,39F8.5)
3000 FORMAT(A80,/)                                                       
3010 FORMAT(1X,'LATTICE CONST.=',3F8.4,3X,'FERMI ENERGY=',F10.5)       
3020 FORMAT(I5,' < NMAT <',I5,3X,'SPIN=',I1,3X,'NAT=',I3, 6x,'SO',i2,' KLmax',i3)
3031 FORMAT(1X,'JATOM',I3,2X,'MULT=',I2,60x)                                 
3030 FORMAT(1X,'JATOM',I3,2X,'MULT=',I2,2x,'ISPLIT=',i2,2x,a)              
3040 FORMAT(1X,'BAND:',I4)                                          
800 FORMAT(////,30X,50(1H-),/,33X,'S T R U C T U R A L   ', 'I N F O R M A T I O N',/,30X,50(1H-),//)
805 FORMAT(3X,'SUBSTANCE',20X,'= ',A80,/)
810 FORMAT(3X,'LATTICE',22X,'= ',A4)
820 FORMAT(3X,'LATTICE CONSTANTS ARE',8X,'= ',3F12.7)
840 FORMAT(3X,'NUMBER OF ATOMS IN UNITCELL  = ',I3)
850 FORMAT(3X,'MODE OF CALCULATION IS',7X,'= ',A4)
1003 FORMAT(A4)
1004 FORMAT(2A5)
2000 FORMAT(//,3X,'=====>>> CPU TIME SUMMARY',/)
2010 FORMAT(12X,'TOTAL       : ',F8.1)
9000 FORMAT('can''t open definition file ',A40)
9010 FORMAT('can''t open unit: ',I2)
9020 FORMAT('       filename: ',A50)
9030 FORMAT('         status: ',A,'  form: ',A)
9040 FORMAT('Error reading file: ',A47)

5100 FORMAT(A10,4I5,3F5.2,A3)
5101 FORMAT(A10,4I10,3F5.2,A3)


END PROGRAM DMFTMAIN
