SUBROUTINE L2MAIN(Qcomplex,nsymop,Qident,mode,projector,Qrenormalize,fUdmft)
  !------------------------------------------------------------------------------
  !-- Main routine for setting up DMFT transformation to local basis set       --
  !-- The key object, DMFTransform can transform self-energy to Kohn-Sham base --
  !-- as well as project the Kohn-Sham green's function to localized base      --
  !------------------------------------------------------------------------------  
  !--  mode: 'g' -> computes the green's function
  !--        'e' -> eigenvalues are printed
  !--        'u' -> printing transformation
  !--        'x' -> computes xqtl2, just for debugging
  USE param,   ONLY: LMAX2, LOMAX, NRAD, nloat, nrf, nmat, nume, gamma, gammac, aom_default, bom_default, nom_default, IBLOCK, matsubara, Cohfacts
  USE struct,  ONLY: VOL, RMT, iord, iz, nat, mult, pos, tau, rotij, tauij, nsym, iatnr, aname
  USE case,    ONLY: cf, csize, maxsize, nl, cix, ll, Sigind, iatom, legend, maxdim, natom, ncix, shft, isort, crotloc, ifirst
  USE sym2,    ONLY: tmat, phase, idet
  USE com,     ONLY: MINWAV, MAXWAV, iso, iso2, emin, emax, EF
  USE abc,     ONLY: KX, KY, KZ, BKX, BKY, BKZ, E, A, ALM, ALML
  USE kpts,    ONLY: mweight, tweight
  USE w_atpar, ONLY: w_alo, w_nlo, w_nlov, w_nlon, w_ilo, w_loor, w_lapw, w_ri_mat, w_P, w_DP, w_jatom, w_FJ, w_DFJ, w_allocate, w_deallocate
  USE com_mpi, ONLY: nprocs, myrank, master, FilenameMPI, Gather_MPI, Reduce_MPI, FindMax_MPI, cpuID
  USE kpts,    only: numkpt !, MKNAME, MIPGR, MWEIGHT, MSX, MSY, MSZ 
  IMPLICIT NONE
  !------- Input variables
  LOGICAL, intent(in)      :: Qcomplex, Qident, Qrenormalize
  INTEGER, intent(in)      :: nsymop, projector
  CHARACTER*1, intent(in)  :: mode
  CHARACTER*200, intent(in):: fUdmft
  !------ External Functions
  INTEGER :: CountSelfenergy
  !------ Common Blocks
  COMMON /GENER/   BR1(3,3), BR2(3,3)
  REAL*8        :: BR1, BR2
  COMMON /XA/      R(NRAD), BK(3), BKROT(3), BKROT2(3), BKROT3(3), BKRLOC(3)
  REAL*8        :: R, BK, BKROT, BKROT2, BKROT3, BKRLOC
  common /loabc/   alo(0:lomax,2,nloat,nrf)                        ! changed by atpar
  REAL*8        :: alo
  common /lolog/   nlo, nlov, nlon, loor(0:lomax), ilo(0:lomax), lapw(0:lmax2) ! changed by atpar
  INTEGER       :: nlo, nlov, nlon, ilo
  logical       :: loor, lapw
  COMMON /RINTEG/  RI_MAT(0:lmax2,0:lmax2,nrf,nrf,2,2)             ! changed by atpar
  REAL*8        :: RI_MAT
  COMMON /ATSPDT/  P(0:LMAX2,2,nrf), DP(0:LMAX2,2,nrf)             ! changed by atpar
  REAL*8        :: P, DP
  !------- allocatable local varaibles
  complex*16, allocatable :: DMFTrans(:,:,:,:,:), STrans(:,:,:,:), GTrans(:,:,:), DMFTU(:,:,:)
  complex*16, allocatable :: xqtl2(:,:,:,:), gk(:), gij(:,:), gloc(:,:), gtot(:), gmk(:,:,:), gmloc(:,:,:,:), Ekp(:,:,:,:)
  complex*16, allocatable :: sigma(:,:,:), Sigc(:,:,:,:), Deltac(:,:,:), Glc(:,:,:), zek(:), s_oo(:,:)
  real*8,     allocatable :: a_real(:), omega(:), Eimpc(:,:), Eimp(:,:), Eimpk(:,:), Olapc(:,:)
  real*8,     allocatable :: Olap(:,:), Olapk(:,:), Cohf0(:)
  complex*16, allocatable :: Eimpm(:,:,:), Eimpmk(:,:,:), Cohf(:,:), evl(:,:), evr(:,:)
  complex*16, allocatable :: Olapm(:,:,:), Olapmk(:,:,:), Olapm0(:,:,:)
  integer,    allocatable :: noccur(:,:), nbandsk(:), nemink(:)
  integer,    allocatable :: cixdim(:), iSx(:,:), nindo(:), csort(:), cix_orb(:)
  complex*16, allocatable :: tEk(:,:,:,:), cfX(:,:,:,:)
  integer,    allocatable :: tnbands(:), tnemin(:)     
  !------- other local arrays
  complex*16  :: h_yl(2*LMAX2+1,iblock)
  complex*16  :: h_alyl(2*LMAX2+1,iblock,2)
  complex*16  :: h_blyl(2*LMAX2+1,iblock,2)
  complex*16  :: YL((LMAX2+1)*(LMAX2+1))
  integer     :: iorbital(natom,lmax2+1)
  !------ other local variables
  logical     :: QoffDiag, Tcompute
  character*10:: BNAME
  complex*16  :: PHSHEL, CFAC, PH_SPIN(2), csum, cvalue, Sigmaij, xomega
  real*8      :: dummy, PI, TWOPI, EMIST, S, T, Z, exxx, ARG1, ARG2, ARG3, ARGT, ARGT2, FAC
  integer     :: norbitals, cnemin, cnbands
  integer     :: lcase1, l1, N1, nidn, ndands, idt, num1, num2, num, iscf, maxdim2
  integer     :: m1, lms1, ind1, is2, m2, lms2, ind2, irf1, irf2
  integer     :: nomega, icix, LATEQ, JC
  integer     :: i, ii, jj, j, iom, icase, lcase, is, is1, itape, jtape, jatom, i3, lda, ldb, ldc, irf, JR, INDEX
  integer     :: iorb, L, nind, ip, iq, it, ioccur, ic, ikp, iikp, N, NE, NEMIN, NEMAX, nbands, isym, M, IND_YL, ibb, LATOM
  integer     :: fh_sig, fh_dos, fh_gc, fh_dt, fh_eig 
  integer     :: wat, iat, im, isrt, iucase, wfirst, wicase, pr_proc, max_bands, natm, maxucase, iorb1, nind1
  integer     :: fh_p, numk, ind, lfirst, inum
  character*100 :: filename
  logical     :: pform
  !
  complex*16  :: IMAG, gtc
  real*8, PARAMETER       :: Ry2eV = 13.60569193
  !------------------------------------------------------------------
  DATA IMAG/(0.0D0,1.0D0)/
  !------------------------------------------------------------------     
  QoffDiag = .FALSE.   !--- if off-diagonal ri_mat overlap matrix is needed (between different l-s) ---!
  fh_sig = 80
  fh_dos = 100
  fh_gc  = 120
  fh_dt  = 140
  fh_eig = 180
  !------ Some parameters which should be read from the input! Not yet implemented! -----!
  gammac = gammac/Ry2eV !--- broadening for correlated orbitals to Ry --!
  gamma  = gamma/Ry2eV  !--- broadening for all orbitals to Ry ---------!
  
  !----------  Some old stuff ---------------------
  PI=ACOS(-1.0D0)
  TWOPI=2.D0*PI
  MAXWAV=0
  MINWAV=100000

  tmat(:,:,:iord) =  iz(:,:,:iord)

  natm = sum(mult) 
  ALLOCATE( csort(nat) )
  CALL Create_Atom_Arrays(csort, maxucase, isort, mult, iatom, nat, natm, natom)

  !----------- Find index to atom/L named iorbital(icase,lcase) -------------------------------!
  CALL Create_Orbital_Arrays(iorbital, norbitals, maxdim2, nl, ll, cix, natom, lmax2, ncix, iso)
  
  allocate( noccur(maxsize,ncix) )
  ALLOCATE( cixdim(ncix), iSx(maxdim2, norbitals), nindo(norbitals), cix_orb(norbitals), cfX(maxdim2,maxdim2,norbitals,norbitals) )
  cfX=0
  
  CALL Create_Other_Arrays(cixdim, iSx, noccur, nindo, cix_orb, cfX, CF, nl, ll, cix, iorbital, csize, Sigind, iso, natom, maxdim, lmax2, ncix, maxsize, norbitals, maxdim2)
  
  if (mode.EQ.'u') CALL PrintSomeArrays('BasicArrays.dat', nat, iso, norbitals, ncix, natom, numkpt, nmat, nume, Qcomplex, lmax2, maxdim2, maxdim, maxsize, nindo, cixdim, nl, ll, cix, iorbital, csize, iSx, Sigind, EF, VOL)

  !-------------------------------------------------------------------------------------!
  !--------------- Reading potential parameters for all atoms.   -----------------------!
  !-- It stores  necessary parameters and uses them below, when looping over atoms -----!
  !-- The step is necessary because green's function of each orbital depends on the ----!
  !--  self-energy of all l's and all atoms. If more than one atom is correlated, we ---!
  !--  can not avoid looping over all correlated atoms in the inside loop --------------!
  !-------------------------------------------------------------------------------------!
  do is=1,iso2 !------ preparing input files case.vsp and case.vspn containing potential parameters -----!
     jtape=17+is
     READ(jtape,2032) ISCF
  end do
  
  call w_allocate(maxucase) !------ contains all variables which depend on atom / which are changed by atpar -----------!


  DO jatom=1,nat    !------  over all atoms (all sorts) even if we do not require qtl for them ------------------!
     iucase = csort(jatom) !--- this particular atom is requested in the input under index iucase --!
     if(iucase.gt.0) then                 !--- requested in the inut ----!
        write(6,'(A,1x,I2,1x,A,1x,I2)')' Calculation for atom', jatom, 'which has iucase=', iucase
        w_jatom(iucase) = jatom
     else
        write(6,'(A,1x,I2,1x,A)')' ATOM=', jatom, ' left out ' !-- not requested in the input ----!
     endif
     
     !----  mult(jatom) stands for multiplicity of each atom -> number of equivalent atoms of this sort --------!
     !--- Reading radial functions U(R), UE(R), for all atoms including those that are left out ----------------!
     do is=1,iso
        itape=8+is
        jtape=17+is     
        rewind(itape)
        if(is.eq.2.and.iso2.eq.1) then  ! fix for SO-coupling and non-spinpolarized
           jtape=18
           rewind(jtape)
           READ(jtape,2032) ISCF 
        endif
        CALL ATPAR(jatom,itape,jtape,is,iso,QoffDiag)
        rewind(itape)
     end do
          
     if (iucase.gt.0) then ! save values of potential parameters
        write(6,'(A,I2,A,I2)')'Saving potential parameters for atom',jatom, ' into ', iucase
        w_alo(:,:,:,:,iucase) = alo(:,:,:,:)
        w_nlo(iucase)  = nlo
        w_nlov(iucase) = nlov
        w_nlon(iucase) = nlon
        w_loor(:,iucase) = loor(:)
        w_ilo(:,iucase)  = ilo(:)
        w_lapw(:,iucase) = lapw(:)
        w_ri_mat(:,:,:,:,:,:,iucase) = ri_mat(:,:,:,:,:,:)
        w_p(:,:,:,iucase) = P(:,:,:)
        w_dp(:,:,:,iucase) = DP(:,:,:)
     endif
  ENDDO
  
  if (Qrenormalize) then
     allocate(Olapm0(maxdim,maxdim,ncix) )
     CALL cmp_overlap(projector,Olapm0, Qcomplex, nsymop, csort, iorbital, cix_orb, nindo, cixdim, iSx, noccur, cfX, maxucase, maxdim2, norbitals)
  endif

  
  !---------------- Reading self-energy ------------------------------------------------------------------!
  !---------------- Input self-energy must be in eV units. Will convert to Rydbergs below ----------------!
  !---------------- Resulting gloc written to file is also in eV units -----------------------------------!
  if (ncix.gt.0) then
     nomega = CountSelfenergy(fh_sig+1, csize(1))-1 !--- How many frequency point exists in the input file? ---!
  else                                           !--- For non-correlated case we do not have self-energy ---!
     nomega = nom_default                        !--- using default ----------------------------------------!
  endif                                        
  
  allocate( sigma(maxsize,ncix,nomega), omega(nomega), s_oo(maxsize,ncix) ) !----- allocating necessary arrays for self-energy ---!

  if (ncix.eq.0) then
     do iom=1,nomega
        omega(iom) = aom_default + (bom_default-aom_default)*iom/(nomega-1.)     !-- default frequency mesh --!
     enddo
  endif
  
  !----------- Actual reading of the self-energy -------------!
  if (mode.eq.'g' .or. mode.eq.'e') then
     WRITE(6,*) '------------------------------------------------------------------------------'
     WRITE(6,*) '----------------- SELF ENERGY AND ITS STRUCTURE ------------------------------'
     do icix=1,ncix
        CALL ReadSelfenergy(fh_sig+icix, sigma(:,icix,:), omega, s_oo(:,icix), gammac, csize(icix), nomega, maxsize)
     enddo
     !--------------  change from eV to Rydbergs --------------------------!
     omega(:) = omega(:)/Ry2eV
     sigma(:,:,:) = sigma(:,:,:)/Ry2eV
     s_oo(:,:) = s_oo(:,:)/Ry2eV
  endif
  !---------------- Reading self-energy --------------------------------!

  

!!! ---------- Preparation of arrays for paralel executaion --------------                                                     
  pr_proc  = floor(numkpt/DBLE(nprocs)+0.999)  ! The maximum number of points calculated per processor                                          
  WRITE(6,'(A,I3,2x,A,I3)') 'pr_proc=', pr_proc, 'tot-k=', numkpt
 
  !------------ allocating some important arrays --------------!
  ALLOCATE( a_real(nmat) ) !---  for eigenvectors -------------!

  if (mode.EQ.'g') then
     allocate( gmk(maxdim,maxdim,ncix), gmloc(maxdim, maxdim, ncix, nomega) )
     gmloc=0
     gmk=0
     allocate(Olapm(maxdim,maxdim,ncix), Olapmk(maxdim,maxdim,ncix))
     Olapm=0
     Olapmk=0
     allocate(Eimpm(maxdim,maxdim,ncix), Eimpmk(maxdim,maxdim,ncix))
     Eimpm=0
     Eimpmk=0
     allocate( gloc(norbitals,nomega), gk(norbitals) ) !--- local green's function and momentum dependent green's function --!
     gloc=0
     gk=0
     allocate(gtot(nomega))
     gtot=0
  elseif (mode.EQ.'e') then
     allocate(Ekp(nomega,nsymop,pr_proc,nume), nbandsk(pr_proc), nemink(pr_proc))
     Ekp=0
     nbandsk=0
     nemink=0
  elseif (mode.EQ.'u') then
     print *, 'cpuID=', cpuID
     filename = TRIM(fUdmft)//ADJUSTL(TRIM(cpuID))  ! fUdmft comes from def file
     fh_p = 399
     pform = .False.
     if (pform) then
        open(fh_p,file=TRIM(filename),status='unknown', form='formatted')
     else 
        open(fh_p,file=TRIM(filename),status='unknown', form='unformatted')
     endif

     numk = pr_proc
     if ((myrank+1)*pr_proc .gt. numkpt) then
        if (numkpt-myrank*pr_proc.gt.0) then
           numk = numkpt-myrank*pr_proc
        else
           numk = 0
        endif
     endif
     if (pform) then
        WRITE(fh_p,*) numk, nsymop, norbitals
        WRITE(fh_p,*) (nindo(iorb), iorb=1,norbitals)
     else
        WRITE(fh_p) numk, nsymop, norbitals
        WRITE(fh_p) (nindo(iorb), iorb=1,norbitals)
     endif

  endif
  if (Cohfacts) ALLOCATE( Cohf0(nume) )
  
  !------------ vector files need to be read from the beginning -------------------!
  do is=1,iso
     rewind(8+is) !--- both vector files: 9 and 10 rewind -------------------------!
  end do
  DO I=1,NAT
     do is=1,iso
        itape=8+is
        READ(itape) EMIST !---- At the beginninge we have linearization energies --!
        READ(itape) EMIST !---- We just skip them ---------------------------------!
     enddo
  ENDDO
  !------------ vector files need to be read from the beginning -------------------!

  
  !!! start-kpoints
  DO ikp=1,numkpt   !------ Over all irreducible k-points ------!
     !--- We need to go over all k-points even though we will compute only some of them on this processor.
     !--- This is because we need to read vector file sequentially.
     iikp = ikp-myrank*pr_proc            ! The index of the point to compute. If negative, do not compute! 
     if (iikp.GT.pr_proc) EXIT            ! Processor finished. The rest of the points will be taken care of by the other processors.
     Tcompute=.FALSE.
     if (iikp.gt.0) Tcompute=.TRUE.       ! If Tcompute is true, the point needs to be computed.

     !------- reading from vector for both spins -----------!
     DO is=1,iso    !------ over up/dn ---------------------!
        itape=8+is
        READ(itape,END=998) S,T,Z,BNAME,N,NE  !--- Here we can jupm out of loop 4 and stop at 998 -----------------------------------!
        IF(N.GT.MAXWAV) MAXWAV=N                                         
        IF(N.LT.MINWAV) MINWAV=N                                         
        READ(itape) (KX(I),KY(I),KZ(I),I=1,N) !--- Reads all reciprocal vectors -----------------------------------------------------!
        DO I=1,N                              !----- Over reciprocal vectors --------------------------------------------------------!
           BKX(I)=(S+KX(I))                   !----  Creates K+k where K is reciprocal vector and k is irreducible k-point ----------!
           BKY(I)=(T+KY(I))
           BKZ(I)=(Z+KZ(I))
        ENDDO
        
        NEMIN=1
        NEMAX=0
        DO 
           READ(itape) NUM,exxx                !----- eigenvalues read -----!
           E(NUM)=exxx
           if (.not.Qcomplex) then
              READ(itape) (A_real(I),I=1,N)    !----- eigenvector read -----!
              do i=1,n
                 a(i,num,is)=(1.d0,0.d0)*a_real(i)
              end do
           else
              READ(itape) (A(I,NUM,is),I=1,N)  !--- eigenvector complex of size n (where n is number of reciprocal vectors) --!
           end if
           IF(E(NUM).LT.EMIN) NEMIN=NEMIN+1
           IF(E(NUM).LT.EMAX) NEMAX=NEMAX+1
           IF(NUM.EQ.NE) EXIT
        ENDDO

        IF (Tcompute) THEN
           DO iucase=1,maxucase
              !--- Computes Bessel functions for all needed l's and atoms. --------------------------------!
              !---  For optimization purposes, done only for irreducible k-points -------------------------!
              !---  output: FJ, DFJ / Spherical Bessel functions and derivative, uses common block BR1/general ---!
              CALL HARMON(N,BKX,BKY,BKZ,LMAX2,w_FJ(:,:,iucase),w_DFJ(:,:,iucase),RMT(w_jatom(iucase))) 
           ENDDO
        ENDIF
     ENDDO  !
     if (.not.Tcompute) CYCLE  ! This k-points was read, but will not be computed on this processor
     !---  Finished reading eigenvectors and eigenvalues for both spins -----!
     !---   E(iband) contains eigenvalues   ---------------------------------!
     !---   A(:,iband,is) contains eigenvectors -----------------------------!





     !do is=1,iso
     !   do num=nemin,nemax
     !      do i=1,N-(nlo+nlon+nlov)
     !         WRITE(*,'(3I3,1x,2f20.14)') is, num, i, a(i,num,is)
     !      enddo
     !   enddo
     !enddo
              


     
     nbands = nemax-nemin+1
     allocate( DMFTrans(nbands,nbands,maxdim2,maxdim2,norbitals) )
     allocate( DMFTU(nbands,maxdim2,norbitals) )
     allocate( gij(nbands,nbands) )
     if (mode.EQ.'e') then
        nbandsk(iikp) = nbands
        nemink(iikp) = NEMIN
        allocate( zek(nbands) )
     elseif (mode.EQ.'x') then
        allocate( xqtl2(nbands,maxdim2,maxdim2,norbitals) )
        xqtl2=0
     elseif (mode.EQ.'u') then
        if (pform) then
           WRITE(fh_p,*) ikp, nbands,maxdim2,norbitals, nemin
        else
           WRITE(fh_p) ikp, nbands,maxdim2,norbitals, nemin
        endif
     endif
     
     if (Cohfacts) then
        ALLOCATE(Cohf(nbands,nbands), evl(nbands,nbands), evr(nbands,nbands))
        READ(1001,*) jj, cnemin, cnbands
        DO i=1,cnbands
           READ(1001,*) (Cohf0(j),j=1,cnbands)
           if (i+cnemin-1.ge.nemin .and. i+cnemin-1.le.nemax) then
              Cohf(i+cnemin-nemin,:) = Cohf0(nemin-cnemin+1:nemax-cnemin+1)
           endif
        ENDDO
        OPEN(1002,FILE='cohfactorsd.dat',status='unknown')
     endif


     allocate( STrans(maxsize,ncix,nbands,nbands) )  ! WWW
     allocate( GTrans(nbands,nbands,norbitals) )
     
     !----------- sum over all k-points in the star of the irreducible k-point ---------------------------------!
     DO isym=1,nsymop  !-- Over all symmetry operarations -> all k-points in the start of irreducible k point --!
              
        DO is=1,iso
           PH_SPIN(is)=EXP((2*is-3)*IMAG*PHASE(isym)/2)  !-- simple phase factor for this k-point ----!
        END DO

        DMFTU=0
        DO icase=1,natom  !--------------- over all atoms requested in the input ------------------------!
           latom = iatom(icase)   ! The succesive number of atom (all atoms counted)
           jatom = isort(latom)   ! The sort of the atom  ( == w_jatom(iucase) )
           lfirst = ifirst(latom)
           iucase = csort(jatom)  ! The renumbert sorts, such that the required atoms from the input give continuous index
           !----  Setting all values of common-blocks which depend on atom and were computed by atpar ---!
           alo(:,:,:,:) = w_alo(:,:,:,:,iucase)
           nlo = w_nlo(iucase)                 
           nlov = w_nlov(iucase)               
           nlon = w_nlon(iucase)               
           loor(:) = w_loor(:,iucase)          
           ilo(:) = w_ilo(:,iucase)            
           lapw(:) = w_lapw(:,iucase)          
           ri_mat(:,:,:,:,:,:) = w_ri_mat(:,:,:,:,:,:,iucase)
           P(:,:,:) = w_p(:,:,:,iucase)        
           DP(:,:,:) = w_dp(:,:,:,iucase)      

           FAC=4.0D0*PI*RMT(jatom)**2/SQRT(VOL)
           
           do lcase=1,nl(icase) !----------- loop over L(jatom) requested in the ionput ---------------!
              l=ll(icase,lcase) !------ current L --!
              CFAC=IMAG**L      !------  (i)^l -----!
              ALM = 0.0         !------  ALM(m,band,nrf,is) will hold product of eigenvectors and a/b expansion coefficients --!
              
              !--------- blocks are for efficiency. Matrix is multiplied in block form. This must be important in the past, while modern BLAS should do that better. I think it is obsolete.
              DO ii=1,N-(nlo+nlon+nlov),iblock !------ iblock is 128 for 32-bit system -------!
                 !-------- nlo-number of local orbitals -----!
                 i3=0                   
                 do i=ii,min(ii+iblock-1,N-(nlo+nlon+nlov))  ! 121
                    !---------  rotates ylm(k+K) to ylm(k'+K) where k' is in star of irreducible k. ------------!
                    i3=i3+1
                    BK(1)=BKX(I) !-----  reciprocal vector and irreducible vector: G=K+k ----!
                    BK(2)=BKY(I)
                    BK(3)=BKZ(I)
                    ! BKROT = R_a.(k+K) transforms to the reducible k-point
                    CALL ROTATE (BK,TMAT(1,1,isym),BKROT)  
                    ! BKROT2 = R_n.R_a.(k+K), transformation from the first atom to an equivalent atom 
                    CALL ROTATE (BKROT, rotij(1,1,latom), BKROT2)
                    !---- BR1 transforms integer reciprocal lattice vectors, as given in the VECTORLIST of LAPW1, into cartesian system ----!
                    ! BKROT3 = R_n.R_a.(k+K), but in cartesian coordinate system
                    BKROT3(1)=BKROT2(1)*BR1(1,1)+BKROT2(2)*BR1(1,2)+BKROT2(3)*BR1(1,3) 
                    BKROT3(2)=BKROT2(1)*BR1(2,1)+BKROT2(2)*BR1(2,2)+BKROT2(3)*BR1(2,3)
                    BKROT3(3)=BKROT2(1)*BR1(3,1)+BKROT2(2)*BR1(3,2)+BKROT2(3)*BR1(3,3)
                    !---- BKRLOC = crotloc.R_n.R_a.(k+K),  rotates according to the user specified local coordinate system.
                    CALL ROTATE (BKROT3,crotloc(1,1,icase),BKRLOC)
                    !---- YLM = Y_{L}(Rotloc.R_g.(k+K))
                    CALL YLM (BKRLOC,LMAX2,YL)  ! 
                    ! (R_n.R_a.(k+K)) *  R(first) * 2pi
                    ARG1=BKROT2(1)*(POS(1,lfirst)+shft(icase,1))*TWOPI
                    ARG2=BKROT2(2)*(POS(2,lfirst)+shft(icase,2))*TWOPI
                    ARG3=BKROT2(3)*(POS(3,lfirst)+shft(icase,3))*TWOPI
                    ! ARGT = (k+K)*tau(isym) * 2pi
                    ARGT =(BKX(I)*TAU(1,isym)+BKY(I)*TAU(2,isym)+BKZ(I)*TAU(3,isym))*TWOPI
                    ! ARGT2 = (R_a.(k+K)).tau_n * 2pi
                    ARGT2=(BKROT(1)*tauij(1,latom)+BKROT(2)*tauij(2,latom)+BKROT(3)*tauij(3,latom))*TWOPI                    
                    ! PHSEHL = e^{I*2pi*( (R_a.(k+K))*tau_n + (K+k)*tau(isym) + (R_n.R_a.(k+K)*R(first)))}
                    PHSHEL=EXP(IMAG*(ARG1+ARG2+ARG3+ARGT+ARGT2))
                    DO  M=1,2*L+1
                       IND_YL=M+L*L
                       h_yl(M,i3)=conjg(yl(ind_yl))*phshel !----- h_yl is rotated yl when k is rotated to k' -----!
                       !WRITE(*,'(7I2,1x,2f20.15)') ikp, icase, is, ii, i, lcase, M, h_yl(M,i3)
                    END DO
                 enddo
                 
                 DO is=1,iso  !--- over both spins
                    i3=0
                    do i=ii,min(ii+iblock-1,N-(nlo+nlon+nlov))
                       i3=i3+1
                       DO M=1,2*L+1
                          if (lapw(l)) then
                             h_ALYL(m,i3,is)=(w_DFJ(L,I,iucase)*P(l,is,2)-w_FJ(L,I,iucase)*DP(l,is,2))* h_yl(M,i3)*ph_spin(is) ! derivatives of bessel functions and spheric harmonics
                             h_BLYL(m,i3,is)=(w_FJ(L,I,iucase)*DP(l,is,1)-w_DFJ(L,I,iucase)*P(l,is,1))* h_yl(M,i3)*ph_spin(is) 
                          else
                             h_ALYL(m,i3,is)=w_FJ(L,I,iucase)/P(l,is,1)/RMT(jatom)**2*h_yl(M,i3)*ph_spin(is)
                             h_BLYL(m,i3,is)=(0.d0,0.d0)
                          end if
                          !WRITE(*,'(7I2,1x,2f20.15,1x,2f20.15,1x,2f20.15)') ikp, icase, is, ii, lcase, i, M, h_yl(M,i3), h_alyl(m,i3,is), PHASE(isym)
                       END DO
                    enddo
                    ibb=min(iblock,N-(nlo+nlon+nlov)-ii+1)
                    lda=2*LMAX2+1
                    ldc=lda
                    ldb=nmat                    
                    !---- h_alyl(2*lmax+1,iblock,is)  contains rotated Apw's, such that chi(r) = (Apw*u(r) + Bpw*udot(r))*Ylm(r)
                    !---- h_blyl(2*lmax+1,iblock,is)  contains rotated Bpw's
                    !---- A(:N,iband,is)              contains eigenvectors, also named C(k+G,inad,is)
                    !---- alm[2*l+1,iband][irf=1,is] += sum_{iK\in block} h_alyl[2*l+1,iK][is]*A[iK,iband][is]
                    !---- alm[2*l+1,iband][irf=2,is] += sum_{iK\in block} h_blyl[2*l+1,iK][is]*A[iK,iband][is]
                    !---- 
                    !---- The results is:
                    !---- alm[lm,iband][1,is] = sum_G Apw(lm,is,K+G) * C(k+G,iband,is)
                    !---- alm[lm,iband][2,is] = sum_G Bpw(lm,is,K+G) * C(k+G,iband,is)
                    !---- Where C(k+G,iband,is) are eigenvectors, and Apw and Bpw are expansion coefficients defined in Shick et.al., PRB 60, 10763 (1999).
                    call zgemm('N','N',2*l+1,nemax-nemin+1,ibb,(1.d0,0.d0), h_alyl(1,1,is),lda,a(ii,nemin,is),ldb,(1.d0,0.d0), alm(1,nemin,1,is),ldc)
                    call zgemm('N','N',2*l+1,nemax-nemin+1,ibb,(1.d0,0.d0), h_blyl(1,1,is),lda,a(ii,nemin,is),ldb,(1.d0,0.d0), alm(1,nemin,2,is),ldc)
                 ENDDO !----------- over both spins ---------!
              ENDDO    !----------- over iblock -------------!
              

              !do is=1,iso
              !   DO M=1,2*L+1
              !      do num=nemin,nemax
              !         WRITE(*,'(6I3,1x,2f20.15,1x,2f20.15)') ikp, icase, is, l, M, num, alm(m,num,1,is), alm(m,num,2,is)
              !      enddo
              !   ENDDO
              !enddo

              !-------------- Adds localized orbitals to alm. -------------------!
              if (nlo.ne.0) then
                 call lomain (nemin,nemax,lfirst,latom,n,jatom,isym,L,iso,icase)
              end if
              
              DO M=1,2*L+1  
                 DO iNUM=NEMIN,NEMAX 
                    DO is=1, iso    
                       DO irf=1,nrf 
                          ALML(L,M,iNUM,irf,is)=ALM(M,iNUM,irf,is)*FAC*CFAC !--------- FACCFAC = (-1)^l*4.*PI*RMT(JATOM)**2/SQRT(VOL) --------!
                       ENDDO
                    ENDDO
                 ENDDO
              ENDDO
           enddo  !--------- end of atom L loop (alm)  ----------------!
           !----------------  ALML(l,m,iband,ifr,ispin) contains (A*C,B*C) for all l,m,iband,ifr=(1,2),ispin --------!
           
           !---------- Computing the DMFT transformation --------------!
           do lcase=1,nl(icase)
              
              l1=ll(icase,lcase)
              nind=(2*l1+1)*iso
              idt = idet(isym)
              icix = cix(icase,lcase)
              iorb = iorbital(icase,lcase)

              CALL CmpDMFTrans(DMFTrans(:,:,:,:,iorb), DMFTU, alml, ri_mat, projector, cfX, l1, iorb, cix_orb, nindo, iso, nemin, nbands, nind, maxdim2, maxdim, icix, idt, lmax2, nume, nrf, natom, ncix, nrf, norbitals)
              if (mode.EQ.'x') then
                 CALL CmpXqtl(xqtl2(:,:,:,iorb), DMFTrans(:,:,:,:,iorb), nbands, nind, maxdim2)
              endif
              
           enddo
        ENDDO  ! over atom-cases
        !-------- computing correlated green's function --------!

        if (Qrenormalize) CALL RenormalizeTrans(DMFTU, Olapm0, cix_orb, nindo, iSx, nbands, maxdim2, norbitals, maxdim, ncix)
        

        CALL CompressSigmaTransformation2(STrans, DMFTU, Sigind, iSx, cix, csize, iorbital, ll, nl, natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize) ! WWW
        
        IF (mode.EQ.'g') THEN
           CALL CompressGcTransformation4(GTrans, DMFTrans, nbands, nindo, norbitals, maxdim2)
           CALL GetImpurityLevels2(Eimpmk, Olapmk, DMFTU, STrans, E, EF, s_oo, nemin, nume, iSx, iorbital, ll, nl, csize, cix, natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize)
           Olapm(:,:,:) = Olapm(:,:,:) + Olapmk(:,:,:) * (mweight(ikp)/tweight/nsymop) ! proper weight for the reducible k-point
           Eimpm(:,:,:) = Eimpm(:,:,:) + Eimpmk(:,:,:) * (mweight(ikp)/tweight/nsymop) ! proper weight for the reducible k-point
        ELSE IF (mode.EQ.'u') THEN
           if (pform) then
              WRITE(fh_p,*) isym
           else
              WRITE(fh_p) isym
           endif
           DO iorb=1,norbitals
              DO ind=1,nindo(iorb)
                 if (pform) then
                    write(fh_p,*) (DMFTU(i,ind,iorb),i=1,nbands)
                 else
                    write(fh_p) (DMFTU(i,ind,iorb),i=1,nbands)
                 endif
              ENDDO
           ENDDO
        ENDIF

        if (Cohfacts) WRITE(1002,'(A,5I5,2x,A)') '#', ikp, isym, nbands, nemin, nomega, ': ikp, isym, nbands nemin, nomega, kweight'
        
        do iom=1,nomega                !-------------------- over frequency ---------------------------------!
           gij=0                       !-------- band structure part of g^-1 --------------------------------!           
           ! Building of G^-1 or H_LDA
           IF (mode.EQ.'e') THEN         !--- We will only compute eigenvalues and eigenvectors
              DO i=1,nbands
                 gij(i,i) = E(i+nemin-1)  !---   preparing hamiltonian: epsk+sigma
              ENDDO
              ! Adding self-energy
              if (ncix.gt.0) then         !-------- adding self-energy if correlated orbitals exist ------------!
                 CALL AddSigma_optimized2(gij, sigma(:,:,iom), STrans, csize, 1, nbands, ncix, maxsize) ! WWW
              endif

              if (Cohfacts) then
                 CALL eigsys(gij, zek, evl, evr, nbands)
                 call zgemm('N','N', nbands,  nbands,  nbands, (1.d0,0.d0), evl, nbands, Cohf, nbands, (0.d0,0.d0), gij, nbands)
                 call zgemm('N','N', nbands,  nbands,  nbands, (1.d0,0.d0), gij, nbands, evr, nbands, (0.d0,0.d0), evl, nbands)
                 WRITE(1002,'(F14.8,1x)',advance='no') omega(iom)*Ry2eV
                 do i=1,nbands
                    WRITE(1002,'(f14.8,1x,f14.8,2x)',advance='no') evl(i,i)
                 enddo
                 WRITE(1002,*)
              else
                 CALL eigvals(gij, zek, nbands)
              endif
              Ekp(iom,isym,iikp,:nbands) = zek(:nbands)
              
           ELSEIF (mode.EQ.'g') THEN
              IF (matsubara) THEN
                 xomega = omega(iom)*IMAG
              ELSE
                 xomega = omega(iom)
              ENDIF
              DO i=1,nbands
                 gij(i,i) = xomega+EF-E(i+nemin-1)  !-----  g^-1 of the LDA part in band representation ----!
              ENDDO
              ! Adding self-energy
              if (ncix.gt.0) then         !-------- adding self-energy if correlated orbitals exist ------------!
                 CALL AddSigma_optimized2(gij, sigma(:,:,iom), STrans, csize, -1, nbands, ncix, maxsize) ! WWW
              endif
              
              DO i=1,nbands               !-------- adding minimum broadening for all bands -------------------!
                 gij(i,i) = gij(i,i) + (0.d0, 1.d0)*gamma
              ENDDO
              if (ncix.gt.0) then
                 CALL zinv(gij,nbands)    !-------- inversion of matrix to get g -------------------------------!
              else
                 DO i=1,nbands
                    gij(i,i) = 1/gij(i,i) !-------- g is diagonal if no correlations ---------------------------!
                 ENDDO
              endif
              
              CALL CmpGkc2(gmk, gij, DMFTU, iSx, iorbital, ll, nl, cix, natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize)
              gmloc(:,:,:,iom) = gmloc(:,:,:,iom) + gmk(:,:,:)*(mweight(ikp)/tweight/nsymop) ! proper weight for the reducible k-point
              
              ! G not resolved in m-quantum number. It is computed by using wave functions, u,\cdot{u},...
              CALL CmpGknc(gk, gij, GTrans, nindo, norbitals, ncix, nbands)
              gloc(:,iom) = gloc(:,iom) + gk(:)*(mweight(ikp)/tweight/nsymop) ! proper weight for the reducible k-point
              
              ! Total g
              gtc=0
              DO i=1,nbands       ! over bands-1
                 gtc = gtc + gij(i,i)
              ENDDO
              ! The rest of the bands added to total g!
              DO i=1,nemin-1 
                 gtc = gtc + 1/(xomega+EF-E(i) + (0.d0, 1.d0)*gamma*5)
              ENDDO
              ! nemax
              DO i=nemax+1,NUM
                 gtc = gtc + 1/(xomega+EF-E(i) + (0.d0, 1.d0)*gamma*5)
              ENDDO
              gtot(iom) = gtot(iom) + gtc*(mweight(ikp)/tweight/nsymop) 

           ELSEIF (mode.EQ.'x') THEN
              ! do nothing
           ELSEIF (mode.EQ.'u') THEN
              ! do nothing
           ELSE
              print *, 'mode not yet implemented 1!'
           ENDIF
           
        enddo  !--  iom loop

     ENDDO !------  isym: over star of the irreducible k-point ----!
     
     if (Cohfacts) then
        DEALLOCATE(Cohf, evl, evr )
     endif
     deallocate( gij, DMFTrans, DMFTU, STrans, GTrans )
     
     if (mode.EQ.'x') then
        xqtl2(:,:,:,:) = xqtl2(:,:,:,:)/nsymop
        DO icase=1,natom
           IF (.NOT.Qident) WRITE(210,'(A)') 'WARNING: LOCAL TRANSFORMATION IS NOT IDENITY. Xqtl might look different!'
           WRITE(210,'(A6,I3)') 'atom#=', iatom(icase)
           do lcase=1,nl(icase)
              WRITE(210,205) S,T,Z,N,NE,BNAME
              iorb = iorbital(icase,lcase)
              CALL TestXqtlDiff(xqtl2(:,:,:,iorb), E, ll(icase,lcase), nbands, iso, nume, maxdim2)
           enddo
        ENDDO
        deallocate(xqtl2)
     elseif (mode.EQ.'e') then
        deallocate( zek )
     endif
     
     WRITE(*,'(I3,A,1x,I3,1x,A,I4)') myrank, ') Finished k-point number', ikp, 'with #bands=', nbands
  ENDDO !---- over reducible k-point: ikp

998 CONTINUE !---- irreducible k-points end (jump) from reading
  !--- end k-points

  !---- Deallocating some arrays before MPI_Gather
  CALL w_deallocate()  
  DEALLOCATE(a_real)
  if (mode.EQ.'g') then
     deallocate( Olapmk, Eimpmk )
     deallocate( gmk, gk )
  endif

  if (Cohfacts) THEN
     DEALLOCATE(Cohf0)
     CLOSE(1002)
     CLOSE(1001)
  endif

  
  !***************** MPI CALLS ***************************
  if (mode.eq.'g') then
     CALL Reduce_MPI(gloc, gtot, gmloc, Olapm, Eimpm, norbitals, nomega, maxdim, ncix)
  elseif (mode.eq.'e') then
     CALL FindMax_MPI(max_bands, nbandsk, pr_proc)
     !if (myrank.eq.master) then
     ALLOCATE( tEk(nomega,nsymop,pr_proc*nprocs,max_bands) )
     ALLOCATE( tnbands(pr_proc*nprocs), tnemin(pr_proc*nprocs) )
     !endif
     CALL Gather_MPI(tEk, tnbands, tnemin, Ekp, nbandsk, nemink, pr_proc, nprocs, numkpt, nume, max_bands, nomega, nsymop) 
  endif
  !***************** MPI CALLS ***************************
  
  !------------- Printing the calculated local green's function for all requested orbitals in requested shape --------!
  IF (mode.EQ.'g' .and. myrank.eq.master) THEN
     allocate( Deltac(maxsize, ncix, nomega), Glc(maxsize, ncix, nomega), Eimpc(maxsize, ncix), Olapc(maxsize,ncix) )
     
     CALL GetDelta2(Deltac, Glc, Eimpc, Olapc, matsubara, omega, sigma, s_oo, gamma, gmloc, Eimpm, Olapm, Sigind, csize, cixdim, noccur, ncix, maxsize, maxdim, nomega, projector)
     CALL PrintGloc(fh_dos, fh_gc, fh_dt, Glc, gloc, gtot, Deltac, omega, csize, nl, ll, legend, iatom, ncix, nomega, natom, norbitals, maxsize, Ry2eV)
     
     WRITE(*,*) 'Eimp=', Eimpc*Ry2eV
     WRITE(*,*) 'final Z due to interstitals=', Olapc
     
     do icix=1,ncix
        write(180+icix,'(A)',advance='no') 'Ed=['
        do it=1,csize(icix)
           write(180+icix,'(f14.8,A)',advance='no') Eimpc(it,icix)*Ry2eV, ','
        enddo
        write(180+icix,'(A)') ']'
        write(180+icix,'(A)',advance='no') 'Olap=['
        do it=1,csize(icix)
           write(180+icix,'(f14.8,A)',advance='no') Olapc(it,icix), ','
        enddo
        write(180+icix,'(A)') ']'
     enddo
     deallocate( Deltac , Glc, Eimpc, Olapc)
  ELSEIF (mode.EQ.'e' .and. myrank.eq.master) THEN
     ! Print Eigenvalues
     DO ikp=1,numkpt
        nbands = tnbands(ikp)
        nemin = tnemin(ikp)
        DO isym=1,nsymop
           WRITE(fh_eig,'(A,5I5,2x,F23.20,2x,A)') '#', ikp, isym, nbands, nemin, nomega, mweight(ikp)/tweight/nsymop, ': ikp, isym, nbands nemin, nomega, kweight'
           DO iom=1,nomega
              WRITE(fh_eig,'(F19.14,2x)',advance='no') omega(iom)*Ry2eV
              DO i=1,nbands
                 WRITE(fh_eig,'(2E24.16,1x)',advance='no') tEk(iom,isym,ikp,i)*Ry2eV
              ENDDO
              WRITE(fh_eig,*)
           ENDDO
        ENDDO
     ENDDO
     DEALLOCATE( tEk, tnbands, tnemin )
  ENDIF
  !------ Deallocation of memory ---------------

  DEALLOCATE( sigma, omega, s_oo)
  deallocate( noccur )
  deallocate( cixdim, iSx, nindo, cix_orb, cfX )
  deallocate( csort )
  
  if (Qident) close(210)
  
  if (mode.EQ.'g') then
     DEALLOCATE( gloc, gmloc, gtot )
     deallocate( Olapm, Eimpm )
  elseif (mode.EQ.'e') then
     DEALLOCATE(Ekp, nbandsk, nemink)
  endif

  if (Qrenormalize) deallocate( Olapm0 )
  !------ Deallocation of memory ---------------
  
  REWIND(itape)
  
205 FORMAT(/,1X,' K-POINT:',3F8.4,1X,I5,I4,2X,A10)                 
200 FORMAT(i5,' atom population matrix')
201 FORMAT(' L1=',i2,', L2=',i2)
202 FORMAT(' L=',i2)
654 FORMAT(6i3,2f14.8)
655 FORMAT(2f14.8,6i3,' Ms1,Ms2,L1,L2,ML1,ML2')
971 FORMAT(1X,f14.8,2x,10f14.8)                               
2032 FORMAT(50X,I2,//)                                                        
!********************************************************************* *PH*
      
  RETURN                    
950 CALL OUTERR('l2main','error reading parallel vectors')
999 CONTINUE
  STOP 'L2main - Error'
!
END SUBROUTINE L2MAIN

! isort, csort, maxucase, natm

SUBROUTINE Create_Atom_Arrays(csort, maxucase, isort, mult, iatom, nat, natm, natom)
  IMPLICIT NONE
  INTEGER, intent(out) :: csort(nat), maxucase
  INTEGER, intent(in)  :: isort(natm), mult(nat), iatom(natom), nat, natm, natom
  !local variables
  INTEGER :: im, iat, wfirst, wat, icase, wicase, iucase
  !wfirst  = 1          ! first atom of this particular sort
  !do iat=1,nat         ! over all sorts?????
  !   !do im=1,MULT(iat) ! over all atoms of this sort
  !   !   wat = wfirst + im-1  ! atom number
  !   !   !isort(wat)=iat       ! sort of each atom
  !   !   icase=0        ! if this atom is requested in the input, it finds its icase
  !   !   do wicase=1,natom
  !   !      if (iatom(wicase).eq.wat) then
  !   !         icase=wicase
  !   !         EXIT
  !   !      endif
  !   !   enddo
  !   !enddo
  !   wfirst = wfirst + MULT(iat)
  !enddo
  
  csort=0
  maxucase=1
  do icase=1,natom
     if (csort(isort(iatom(icase))).EQ.0) then
        csort(isort(iatom(icase))) = maxucase
        maxucase = maxucase + 1
     endif
  enddo
  maxucase = maxucase-1

  WRITE(6,*)
  WRITE(6,*) '********** Information about the atomic index arrays ************'
  WRITE(6,*) 'iatom=', iatom
  WRITE(6,*) 'isort=', isort
  WRITE(6,*) 'csort=', csort
  WRITE(6,*) 'maxucase=', maxucase
  
  DO icase=1,natom
     iucase = csort(isort(iatom(icase)))
     WRITE(6, '(A,6I6,1x)') 'icase, iucase, jatom, latom=', icase, iucase, isort(iatom(icase)), iatom(icase)
  ENDDO

END SUBROUTINE Create_Atom_Arrays

SUBROUTINE Create_Orbital_Arrays(iorbital, norbitals, maxdim2, nl, ll, cix, natom, lmax2, ncix, iso)
  IMPLICIT NONE
  INTEGER, intent(out) :: iorbital(natom,lmax2+1), maxdim2, norbitals
  INTEGER, intent(in)  :: nl(natom), ll(natom,4), cix(natom,4), natom, lmax2, ncix, iso
  !----- locals
  INTEGER :: icase, lcase, l1, nind, icix
  !----------- Find index to atom/L named iorbital(icase,lcase) -------------------------------!
  iorbital=0
  maxdim2=0   !-- maximum size of the matrix over all atoms and l's requested in the input ----!
              !---- do not confuse with maxdim, which is maximum size for correlated orbitals -!
  norbitals=0 !-- number of atom/l cases, called orbitals -------------------------------------!
  do icase=1,natom
     do lcase=1,nl(icase)
        norbitals = norbitals+1
        iorbital(icase,lcase)=norbitals  !-- index to the orbital number ----!
        icix = cix(icase,lcase)
        
        l1=ll(icase,lcase)
        nind=(2*l1+1)*iso
        maxdim2 = max(maxdim2, nind )

     enddo
  enddo  
  
  WRITE(6,*)
  WRITE(6,*) '*****  Arrays for correlated blocks ******'
  WRITE(6,*) 'norbitals=', norbitals
  WRITE(6,*) 'maxdim2=', maxdim2
  DO icase=1,natom
     DO lcase=1,nl(icase)
        WRITE(6,'(A,I2,A,I2,A,I4)') 'iorbital(',icase,',',lcase,')=', iorbital(icase,lcase)
     ENDDO
  ENDDO
  DO icase=1,natom
     DO lcase=1,nl(icase)
        WRITE(6,'(A,I2,A,I2,A,I4)') 'cix(',icase,',',lcase,')=', cix(icase,lcase)
     ENDDO
  ENDDO
  !----------- Find index to atom/L named iorbital(icase,lcase) --------
END SUBROUTINE Create_Orbital_Arrays
  

SUBROUTINE Create_Other_Arrays(cixdim, iSx, noccur, nindo, cix_orb, cfX, CF, nl, ll, cix, iorbital, csize, Sigind, iso, natom, maxdim, lmax2, ncix, maxsize, norbitals, maxdim2)
  IMPLICIT NONE
  INTEGER, intent(out)    :: cixdim(ncix), iSx(maxdim2, norbitals), noccur(maxsize,ncix), nindo(norbitals), cix_orb(norbitals)
  COMPLEX*16, intent(out) :: cfX(maxdim2,maxdim2,norbitals,norbitals)
  COMPLEX*16, intent(in)  :: CF(maxdim,maxdim,ncix)
  INTEGER, intent(in)     :: nl(natom), ll(natom,4), cix(natom,4), iorbital(natom,lmax2+1), csize(ncix)
  INTEGER, intent(in)     :: Sigind(maxdim,maxdim,ncix)
  INTEGER, intent(in)     :: natom, maxdim, lmax2, ncix, iso, maxsize, norbitals, maxdim2
  !----- locals
  INTEGER :: icase, lcase, iorb, icix, L, nind, ip, iq, it, ioccur, i, ic, l1, nind1, nind2, ip1, iorb1, iorb2, ind1, ind2

  cixdim=0
  iSx=0
  DO icase=1,natom            
     do lcase=1,nl(icase)     
        icix = cix(icase,lcase)
        l1 = ll(icase,lcase)
        iorb = iorbital(icase,lcase)
        nind1 = (2*l1+1)*iso
        nindo(iorb) = nind1

        cix_orb(iorb) = icix
        
        if ( icix.EQ.0 ) CYCLE
        do ip1=1,nind1
           cixdim(icix) = cixdim(icix) + 1
           iSx(ip1,iorb) = cixdim(icix)
           WRITE(6,'(A,7I5)') 'icase,lcase,icix,iorb,nind1,ip1,iSx=', icase, lcase, icix, iorb, nind1,ip1,cixdim(icix)
        enddo
     enddo
  ENDDO
  noccur=0
  DO icix=1,ncix
     DO ip=1,cixdim(icix)
        DO iq=1,cixdim(icix)
           it = Sigind(ip,iq,icix)
           if (it.gt.0)  noccur(it,icix) = noccur(it,icix) + 1
        ENDDO
     ENDDO
  ENDDO

  DO iorb1=1,norbitals
     nind1 = nindo(iorb1)
     if ( cix_orb(iorb1).EQ.0 ) CYCLE
     DO iorb2=1,norbitals
        nind2 = nindo(iorb2)
        if ( cix_orb(iorb1).NE.cix_orb(iorb2) ) CYCLE
        icix = cix_orb(iorb1)
        do ind1=1,nind1
           do ind2=1,nind2
              !WRITE(*,'(I2,I2,I3,I3,I3,I3,2x,I3,2f14.5)') iorb1, iorb2, ind1, ind2, iSx(ind1,iorb1), iSx(ind2,iorb2), icix, CF( iSx(ind1,iorb1),iSx(ind2,iorb2), icix )
              cfX(ind1,ind2,iorb1,iorb2) = CF( iSx(ind1,iorb1),iSx(ind2,iorb2), icix )
           enddo
        enddo
     ENDDO
  ENDDO
  
  
  do icix=1,ncix
     WRITE(6,'(A,I2,A,I2)') 'cixdim(', icix, ')=', cixdim(icix)
     DO it=1,csize(icix)
        WRITE(6,'(A,I2,A,I2,A,I4)') 'noccur(',it,',',icix,')=', noccur(it,icix)
     ENDDO
  enddo
  do iorb=1,norbitals
     do ip1=1,nindo(iorb)
        WRITE(6,'(A,I2,A,I2,A,I3)') 'iSx(', ip1, ',', iorb, ')=', iSx(ip1,iorb)
     enddo
  enddo
  do iorb=1,norbitals
     WRITE(6,'(A,I2,A,I3)') 'nindo(', iorb, ')=', nindo(iorb)
  enddo
  do iorb1=1,norbitals
     do iorb2=1,norbitals
        if ( cix_orb(iorb1).EQ.0 ) CYCLE
        if ( cix_orb(iorb1).NE.cix_orb(iorb2) ) CYCLE
        WRITE(6,'(A,I2,A,I2)') 'CF for iorb1=', iorb1, ' iorb2=', iorb2
        do ind1=1,nind1
           do ind2=1,nind2
              WRITE(6,'(f14.8,1x,f14.8,3x)', advance='no') real(cfX(ind1,ind2,iorb1,iorb2)), aimag(cfX(ind1,ind2,iorb1,iorb2))
           enddo
           WRITE(6,*)
        enddo
     enddo
  enddo
END SUBROUTINE Create_Other_Arrays


SUBROUTINE PrintSomeArrays(filename, nat, iso, norbitals, ncix, natom, nkpt, nmat, nume, Qcomplex, lmax2, maxdim2, maxdim, maxsize, nindo, cixdim, nl, ll, cix, iorbital, csize, iSx, Sigind, EF, VOL)
  IMPLICIT NONE
  CHARACTER*100, intent(in) :: filename
  INTEGER, intent(in) :: nat, iso, norbitals, ncix, natom, nkpt, nmat, nume, lmax2, maxdim2, maxdim, maxsize
  LOGICAL, intent(in) :: Qcomplex
  INTEGER, intent(in) :: nindo(norbitals), cixdim(ncix), nl(natom), ll(natom,4), cix(natom,4), iorbital(natom,lmax2+1)
  INTEGER, intent(in) :: csize(ncix), iSx(maxdim2,norbitals), Sigind(maxdim,maxdim,ncix)
  REAL*8, intent(in)  :: EF, VOL
  ! locals
  INTEGER :: fh, icase, lcase, iorb, ip1, icix, ip, iq
  fh=996
  open(fh,file=TRIM(filename),status='unknown', form='formatted')
  WRITE(fh, *) 'nat, iso, norbitals, ncix, natom'
  WRITE(fh, *) nat, iso, norbitals, ncix, natom
  WRITE(fh, *) 'nkpt, nmat, nume'
  WRITE(fh, *) nkpt, nmat, nume
  WRITE(fh, *) 'Qcomplex'
  WRITE(fh, *) Qcomplex
  WRITE(fh, *) 'lmax2, maxdim2, maxdim, maxsize'
  WRITE(fh, *) lmax2, maxdim2, maxdim, maxsize
  WRITE(fh, *) 'nindo'
  WRITE(fh, *) (nindo(iorb),iorb=1,norbitals)
  WRITE(fh, *) 'cidim'
  WRITE(fh, *) (cixdim(icix),icix=1,ncix)
  WRITE(fh, *) 'nl'
  WRITE(fh, *) (nl(icase), icase=1,natom)
  WRITE(fh, *) 'll, iorbital, cix, nind'
  do icase=1,natom
     do lcase=1,nl(icase)
        WRITE(fh, *) ll(icase,lcase)
        WRITE(fh, *) iorbital(icase,lcase)
        WRITE(fh, *) cix(icase,lcase)
        WRITE(fh, *) (2*ll(icase,lcase)+1)*iso          ! nind
     enddo
  enddo
  
  WRITE(fh, *) 'csize'
  WRITE(fh, *) (csize(icix), icix=1,ncix)
  
  WRITE(fh, *) 'iSx'
  do iorb=1,norbitals
     do ip1=1,nindo(iorb)
        WRITE(fh, *) iSx(ip1,iorb)
     enddo
  enddo
  WRITE(fh, *) 'Sigind'
  DO icix=1,ncix
     DO ip=1,cixdim(icix)
        DO iq=1,cixdim(icix)
           WRITE(fh, *) Sigind(ip,iq,icix)
        ENDDO
     ENDDO
  ENDDO
  WRITE(fh, *) 'EF, VOL'
  WRITE(fh, *) EF, VOL
END SUBROUTINE PrintSomeArrays




SUBROUTINE CompressSigmaTransformation2(STrans, DMFTU, Sigind, iSx, cix, csize, iorbital, ll, nl, natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize)
  IMPLICIT NONE
  COMPLEX*16, intent(out) :: STrans(maxsize,ncix,nbands,nbands)
  COMPLEX*16, intent(in)  :: DMFTU(nbands,maxdim2,norbitals)
  INTEGER, intent(in)     :: iSx(maxdim2, norbitals)
  INTEGER, intent(in)     :: Sigind(maxdim,maxdim,ncix), cix(natom,4), csize(ncix)
  INTEGER, intent(in)     :: iorbital(natom,lmax2+1), ll(natom,4), nl(natom)
  INTEGER, intent(in)     :: natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize
  !----- locals
  INTEGER    :: i, j, icase, jcase, lcase, l1case, l2case, iorb, iorb1, iorb2, it, ind1, nind1, ind2, nind2, icix, l1, l2, ip1, ip2
  STrans=0
  DO icase=1,natom      
     do l1case=1,nl(icase) 
        icix = cix(icase,l1case)
        if ( icix.EQ.0 ) CYCLE
        l1 = ll(icase,l1case)
        nind1 = (2*l1+1)*iso
        iorb1 = iorbital(icase,l1case)
        DO jcase=1,natom
           do l2case=1,nl(jcase)
              if ( cix(jcase,l2case).NE.icix ) CYCLE
              l2 = ll(jcase,l2case)
              nind2 = (2*l2+1)*iso
              iorb2 = iorbital(jcase,l2case)
              do ind1=1,nind1
                 do ind2=1,nind2
                    it = Sigind( iSx(ind1,iorb1),iSx(ind2,iorb2), icix )
                    
                    if (it.gt.0) then
                       !WRITE(*,'(A,I2,1x,A,I2,1x,A,I2,1x,A,I2,1x,A,I2,1x,A,I2,1x,A,I2,1x,A,I2,1x,A,I2,1x,A,I2,1x,A,I2)') 'icase=',icase, 'jcase=',jcase, 'l1case=',l1case, 'l2case=',l2case, 'iorb1=',iorb1, 'iorb2=',iorb2, 'ind1=',ind1, 'ind2=',ind2, 'iSx1=', iSx(ind1,iorb1), 'iSx2=', iSx(ind2,iorb2), 'it=',it
                       
                       DO i=1,nbands                     ! over bands-1
                          do j=1,nbands                  ! over bands-2
                             STrans(it,icix,i,j) = STrans(it,icix,i,j) + conjg(DMFTU(j,ind2,iorb2))*DMFTU(i,ind1,iorb1)
                          enddo
                       ENDDO
                       
                    endif
                 enddo
              enddo
           enddo
        ENDDO
     enddo
  ENDDO
END SUBROUTINE CompressSigmaTransformation2


SUBROUTINE AddSigma_optimized2(gij, sigma, STrans, csize, sign, nbands, ncix, maxsize)
  !-- Add's self-energy to inverse of the Green's function in band representation
  !-- 50% of all time is spend in this subroutine, the rest 50 % in CmpGk
  IMPLICIT NONE
  COMPLEX*16, intent(inout):: gij(nbands,nbands)
  COMPLEX*16, intent(in)   :: sigma(maxsize,ncix)
  COMPLEX*16, intent(in)  :: STrans(maxsize,ncix,nbands,nbands)
  INTEGER, intent(in)      :: csize(ncix), sign, nbands, ncix, maxsize
  !----- locals
  COMPLEX*16 :: Sigmaij
  INTEGER    :: i, j, icix, it
  
  DO i=1,nbands                     ! over bands-1
     do j=1,nbands                  ! over bands-2
        Sigmaij=0
        DO icix=1,ncix
           do it=1,csize(icix)
              Sigmaij = Sigmaij + STrans(it,icix,i,j)*sigma(it,icix)
           enddo
        ENDDO
        gij(i,j) = gij(i,j) + sign*Sigmaij
     enddo
  ENDDO
END SUBROUTINE AddSigma_optimized2


SUBROUTINE CompressGcTransformation4(GTrans, DMFTrans, nbands, nindo, norbitals, maxdim2)
  IMPLICIT NONE
  COMPLEX*16, intent(out)  :: GTrans(nbands,nbands,norbitals)
  COMPLEX*16, intent(in)   :: DMFTrans(nbands,nbands,maxdim2,maxdim2,norbitals)
  INTEGER,    intent(in)   :: nbands, nindo(norbitals), norbitals, maxdim2
  !----- locals
  COMPLEX*16 :: csum
  INTEGER    :: i, j, iorb, ind1, nind
  DO i=1,nbands                     ! over bands-1
     do j=1,nbands                  ! over bands-2
        DO iorb=1,norbitals
           nind = nindo(iorb)

           csum=0
           do ind1=1,nind
              csum = csum + DMFTrans(i,j,ind1,ind1,iorb)
           enddo
           GTrans(i,j,iorb) = csum
           
        ENDDO
     enddo
  ENDDO
END SUBROUTINE CompressGcTransformation4



SUBROUTINE CmpGkc2(gmk, gij, DMFTU, iSx, iorbital, ll, nl, cix, natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize)
  IMPLICIT NONE
  COMPLEX*16, intent(out) :: gmk(maxdim,maxdim,ncix)
  COMPLEX*16, intent(in) :: gij(nbands,nbands)
  COMPLEX*16, intent(in) :: DMFTU(nbands,maxdim2,norbitals)
  INTEGER, intent(in)     :: iSx(maxdim2, norbitals)
  INTEGER, intent(in)     :: iorbital(natom,lmax2+1), ll(natom,4), nl(natom), cix(natom,4)
  INTEGER, intent(in)     :: natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize
  ! locals
  COMPLEX*16, allocatable :: tgk(:,:), tmp(:,:)
  INTEGER    :: it, i, j, icase, jcase, l1case, l2case, icix, l1, l2, nind1, nind2, iorb1, iorb2, ind1, ind2
  gmk=0
  allocate( tmp(maxdim2,nbands), tgk(maxdim2,maxdim2) )
  DO icase=1,natom      
     do l1case=1,nl(icase) 
        icix = cix(icase,l1case)
        if ( icix.EQ.0 ) CYCLE
        l1 = ll(icase,l1case)
        nind1 = (2*l1+1)*iso
        iorb1 = iorbital(icase,l1case)
        DO jcase=1,natom
           do l2case=1,nl(jcase)
              if ( cix(jcase,l2case).NE.icix ) CYCLE
              l2 = ll(jcase,l2case)
              nind2 = (2*l2+1)*iso
              iorb2 = iorbital(jcase,l2case)
              !print *, 'iorb1,iorb2=', iorb1, iorb2, maxdim2, nind1, nind2, nbands, maxdim2
              
              call zgemm('C','N', nind1, nbands, nbands, (1.d0,0.d0), DMFTU(:,:,iorb1), nbands, gij(:,:),nbands, (0.d0,0.d0), tmp(:,:),maxdim2)
              call zgemm('N','N', nind1, nind2, nbands, (1.d0,0.d0), tmp,maxdim2, DMFTU(:,:,iorb2),nbands, (0.d0,0.d0), tgk,maxdim2)
              
              !print *, 'tgk1=', tgk(:nind1,:nind2)
              !tgk(:nind1,:nind2) = matmul(matmul(transpose(conjg(DMFTU(:,:nind1,iorb1))), gij),DMFTU(:,:nind2,iorb2))
              !print *, 'tgk2=', tgk(:nind1,:nind2)

              do ind1=1,nind1
                 do ind2=1,nind2
                    gmk( iSx(ind1,iorb1), iSx(ind2,iorb2), icix ) = tgk(ind1,ind2)
                 enddo
              enddo
           enddo
        ENDDO
     enddo
  ENDDO

  deallocate( tmp, tgk )
END SUBROUTINE CmpGkc2


SUBROUTINE CmpGknc(gk, gij, GTrans, nindo, norbitals, ncix, nbands)
  ! Creates local green's function
  ! 50% of all time is spend in this subroutine, the rest 50 % in AddSigma
  IMPLICIT NONE
  COMPLEX*16, intent(out):: gk(norbitals)
  COMPLEX*16, intent(in) :: gij(nbands,nbands)
  COMPLEX*16, intent(in) :: GTrans(nbands,nbands,norbitals)
  INTEGER, intent(in)    :: nindo(norbitals), ncix, nbands, norbitals
  !------ locals
  INTEGER    :: i, j, icase, lcase, icix, l1, nind, iorb
  COMPLEX*16 :: gc

  gk=0
  DO iorb=1,norbitals
     nind = nindo(iorb)
     gc=0
     if (ncix.gt.0) then    ! off-diagonal g in band representation
        DO i=1,nbands                      ! over bands-1
           do j=1,nbands                   ! over bands-2
              gc = gc + gij(i,j)*GTrans(i,j,iorb)
           enddo
        ENDDO
     else ! for non-correlated case, green's function is diagonal in band representation
        DO i=1,nbands      
           gc = gc + gij(i,i)*GTrans(i,i,iorb)
        ENDDO
     endif
     gk(iorb)=gc !/nind
  ENDDO
  
END SUBROUTINE CmpGknc



SUBROUTINE GetImpurityLevels2(Eimpmk, Olapmk, DMFTU, STrans, E, EF, s_oo, nemin, nume, iSx, iorbital, ll, nl, csize, cix, natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize)
  IMPLICIT NONE
  COMPLEX*16, intent(out) :: Eimpmk(maxdim,maxdim,ncix)
  COMPLEX*16, intent(out) :: Olapmk(maxdim,maxdim,ncix)
  COMPLEX*16, intent(out) :: DMFTU(nbands,maxdim2,norbitals), STrans(maxsize,ncix,nbands,nbands)
  REAL*8,     intent(in)  :: E(nume), EF                         ! eigenvalues, chemical potential
  COMPLEX*16, intent(in)  :: s_oo(maxsize,ncix)
  INTEGER, intent(in)     :: nemin, nume, iSx(maxdim2, norbitals)
  INTEGER, intent(in)     :: iorbital(natom,lmax2+1), ll(natom,4), nl(natom), csize(ncix), cix(natom,4)
  INTEGER, intent(in)     :: natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize
  !--------- locals
  COMPLEX*16, allocatable :: tgk(:,:), tmp(:,:), gij(:,:), olp(:,:)
  INTEGER    :: it, i, j, icase, jcase, l1case, l2case, icix, l1, l2, nind1, nind2, iorb1, iorb2, ind1, ind2

  
  allocate( tmp(maxdim2,nbands), tgk(maxdim2, maxdim2), gij(nbands,nbands), olp(maxdim2,maxdim2) )

  gij=0
  DO i=1,nbands
     gij(i,i) = (E(i+nemin-1)-EF)
  ENDDO  
  
  CALL AddSigma_optimized2(gij, s_oo, STrans, csize, 1, nbands, ncix, maxsize)
  
  Eimpmk=0
  Olapmk=0
  DO icase=1,natom      
     do l1case=1,nl(icase) 
        icix = cix(icase,l1case)
        if ( icix.EQ.0 ) CYCLE
        l1 = ll(icase,l1case)
        nind1 = (2*l1+1)*iso
        iorb1 = iorbital(icase,l1case)
        DO jcase=1,natom
           do l2case=1,nl(jcase)
              if ( cix(jcase,l2case).NE.icix ) CYCLE
              l2 = ll(jcase,l2case)
              nind2 = (2*l2+1)*iso
              iorb2 = iorbital(jcase,l2case)

              call zgemm('C','N', nind1, nind2, nbands, (1.d0,0.d0), DMFTU(:,:,iorb1),nbands, DMFTU(:,:,iorb2),nbands, (0.d0,0.d0), olp,maxdim2)
              call zgemm('C','N', nind1, nbands, nbands, (1.d0,0.d0), DMFTU(:,:,iorb1), nbands, gij(:,:),nbands, (0.d0,0.d0), tmp(:,:),maxdim2)
              call zgemm('N','N', nind1, nind2,  nbands, (1.d0,0.d0), tmp,maxdim2, DMFTU(:,:,iorb2), nbands, (0.d0,0.d0), tgk,maxdim2)
              
              do ind1=1,nind1
                 do ind2=1,nind2
                    Eimpmk( iSx(ind1,iorb1), iSx(ind2,iorb2), icix ) = tgk(ind1,ind2)
                    Olapmk( iSx(ind1,iorb1), iSx(ind2,iorb2), icix ) = olp(ind1,ind2)
                 enddo
              enddo
              
           enddo
        ENDDO
     enddo
  ENDDO
  deallocate( gij, tmp, tgk, olp )
END SUBROUTINE GetImpurityLevels2



SUBROUTINE GetDelta2(Deltac, Glc, Eimpc, Olapc, matsubara, omega, sigma, s_oo, gamma, gmloc, Eimpm, Olapm, Sigind, csize, cixdim, noccur, ncix, maxsize, maxdim, nomega, projector)
  IMPLICIT NONE
  COMPLEX*16, intent(out):: Deltac(maxsize, ncix,nomega), Glc(maxsize,ncix,nomega)
  REAL*8, intent(out)    :: Eimpc(maxsize, ncix), Olapc(maxsize,ncix)
  LOGICAL, intent(in)    :: matsubara
  REAL*8,  intent(in)    :: omega(nomega)
  COMPLEX*16, intent(in) :: sigma(maxsize,ncix,nomega), s_oo(maxsize,ncix)                                                                                                   
  REAL*8, intent(in)     :: gamma
  COMPLEX*16, intent(in) :: gmloc(maxdim,maxdim,ncix,nomega)
  COMPLEX*16, intent(inout) :: Eimpm(maxdim,maxdim,ncix), Olapm(maxdim,maxdim,ncix)
  INTEGER, intent(in)    :: Sigind(maxdim,maxdim,ncix), csize(ncix), cixdim(ncix)
  INTEGER, intent(in)    :: noccur(maxsize,ncix)
  INTEGER, intent(in)    :: ncix, maxsize, maxdim, nomega, projector
  !----- locals
  COMPLEX*16, allocatable:: gcx(:,:), Eimpx(:,:), deltx(:,:), sigcx(:,:), sig_oo(:,:)
  COMPLEX*16 :: xomega
  INTEGER    :: icix, ip, iq, it, iom, cixdm
  complex*16 :: IMAG
  !-----------------------
  DATA IMAG/(0.0D0,1.0D0)/
  !-----------------------
  Deltac=0
  Olapc=0
  Glc=0
  Eimpc=0
  DO icix=1,ncix

     cixdm = cixdim(icix)     
     allocate( sigcx(cixdm,cixdm), gcx(cixdm,cixdm), Eimpx(cixdm,cixdm), deltx(cixdm,cixdm), sig_oo(cixdm,cixdm) )
     sig_oo=0
     !---- Olap to vector form, and s_oo to matrix form
     DO ip=1,cixdm
        do iq=1,cixdm
           it = Sigind(ip,iq,icix)
           if (it.gt.0) then
              Olapc(it, icix) =  Olapc(it, icix) + real(Olapm(ip,iq,icix))
              sig_oo(ip,iq) = s_oo(it,icix)
           endif
        enddo
     ENDDO
     DO it=1,csize(icix)
        Olapc(it, icix) = Olapc(it, icix)/noccur(it,icix)
     ENDDO

     if (projector.LT.0) then
        ! Set off-diagonal terms to zero. DMFT-SCC is taken to be purely diagonal
        DO ip=1,cixdm
           do iq=1,cixdm
              if (ip.NE.iq) then
                 Olapm(ip,iq,icix)=0.0
                 Eimpm(ip,iq,icix)=0.0
              end if
           enddo
        enddo
     endif
     
     ! O^{-1} needed to Eimp = O^{-1} H_{loc} O^{-1}
     CALL zinv(Olapm(:cixdm,:cixdm,icix), cixdm)
     Eimpx = matmul( matmul(Olapm(:cixdm,:cixdm,icix),Eimpm(:cixdm,:cixdm,icix)), Olapm(:cixdm,:cixdm,icix)) !-- normalize by overlap
     Eimpx = Eimpx - sig_oo !--- subtracting S_oo from Eimp such that the real impurity level is Eimpc-Edc

     !---- Eimpc to vector form
     DO ip=1,cixdm
        do iq=1,cixdm
           it = Sigind(ip,iq,icix)
           if (it.gt.0) then
              Eimpc(it, icix) =  Eimpc(it, icix) + real(Eimpx(ip,iq))
           endif
        enddo
     ENDDO
     DO it=1,csize(icix)
        Eimpc(it, icix) = Eimpc(it, icix)/noccur(it,icix)
     ENDDO
     
     !DO it=1,csize(icix)
     !   Eimpc(it, icix) = Eimpc(it, icix) - s_oo(it, icix)
     !ENDDO
     
     do iom=1,nomega !------- for all frequencies --!
        
        IF (matsubara) THEN
           xomega = omega(iom)*IMAG
        ELSE
           xomega = omega(iom)
        ENDIF
        !---- unpacking sigma to matrix form
        sigcx=0
        DO ip=1,cixdm
           do iq=1,cixdm
              it = Sigind(ip,iq,icix)
              if (it.gt.0) then
                 sigcx(ip,iq) = sigma( it, icix, iom )
              endif
           enddo
        ENDDO
        
        gcx = gmloc(:cixdm,:cixdm,icix,iom)

        if (projector.LT.0) then
           ! Set off-diagonal terms to zero. DMFT-SCC is taken to be purely diagonal
           DO ip=1,cixdm
              do iq=1,cixdm
                 if (ip.NE.iq) then
                    gcx(ip,iq)=0.0
                 end if
              enddo
           enddo
        endif

        CALL zinv(gcx,cixdm)
        
        !--- DMFT self-conistency condition
        deltx = Olapm(:cixdm,:cixdm,icix)*xomega - Eimpx - gcx - sigcx 
        do ip=1,cixdm
           deltx(ip,ip) = deltx(ip,ip) + (0.d0, 1.d0)*gamma
        enddo
        !---- packing delta to vector form
        DO ip=1,cixdm
           do iq=1,cixdm
              it = Sigind(ip,iq,icix)
              if (it.gt.0) then
                 Deltac( it, icix, iom ) =  Deltac( it, icix, iom ) + deltx(ip,iq)
                 Glc( it, icix, iom ) =  Glc( it, icix, iom ) + gmloc(ip,iq,icix,iom)
              endif
           enddo
        ENDDO
        DO it=1,csize(icix)
           Deltac(it, icix, iom) = Deltac(it, icix, iom)/noccur(it,icix)
           Glc(it, icix, iom) = Glc(it, icix, iom)/noccur(it,icix)
        ENDDO
     enddo

     deallocate( sigcx, gcx, Eimpx, deltx, sig_oo )
  ENDDO
END SUBROUTINE GetDelta2




!SUBROUTINE PrintEigvals0(fh_eig, ikp, isym, nbands, nemin, nomega, xweigh)
!  USE com_mpi, ONLY: myrank, master
!  IMPLICIT NONE
!  INTEGER, intent(in) :: fh_eig, ikp, isym, nbands, nemin, nomega
!  REAL*8, intent(in)  :: xweigh
!  if (myrank.ne.master) return
!  WRITE(fh_eig,'(A,5I4,2x,F10.7,2x,A)') '#', ikp, isym, nbands, nemin, nomega, xweigh, ': ikp, isym, nbands nemin, nomega, kweight'
!END SUBROUTINE PrintEigvals0
!SUBROUTINE PrintEigvals1(fh_eig, zek, xomega, nbands, Ry2eV)
!  USE com_mpi, ONLY: myrank, master
!  IMPLICIT NONE
!  INTEGER, intent(in)    :: fh_eig, nbands
!  COMPLEX*16, intent(in) :: zek(nbands)
!  REAL*8, intent(in)     :: xomega, Ry2eV
!  ! locals
!  INTEGER :: i
!  if (myrank.ne.master) return
!  WRITE(fh_eig,'(F14.10,2x)',advance='no') xomega*Ry2eV
!  DO i=1,nbands
!     WRITE(fh_eig,'(2E16.8,1x)',advance='no') zek(i)*Ry2eV
!  ENDDO
!  WRITE(fh_eig,*)
!END SUBROUTINE PrintEigvals1

!SUBROUTINE CmpGk0(gk, z, E, DMFTrans, nemin, nbands, nind, maxdim2, nume)
!  ! Creates local green's function
!  IMPLICIT NONE
!  COMPLEX*16, intent(out):: gk(maxdim2,maxdim2)  ! green's functions in Kohn-Sham base
!  COMPLEX*16, intent(in) :: z              ! omega+mu
!  REAL*8,     intent(in) :: E(nume)        ! eigenvalues
!  COMPLEX*16, intent(in) :: DMFTrans(nbands,nbands,maxdim2,maxdim2)
!  INTEGER, intent(in)    :: nbands, nind, nume, nemin, maxdim2
!  ! local variables
!  COMPLEX*16 :: csum
!  INTEGER    :: ind1, ind2, i
!  do ind1=1,nind      ! over orbitals
!     do ind2=1,nind   ! over orbitals
!        csum=0
!        do i=1,nbands ! over bands
!           csum = csum + DMFTrans(i,i,ind1,ind2)/(z-E(i+nemin-1))
!        enddo
!        gk(ind1,ind2) = csum
!     enddo
!  enddo
!END SUBROUTINE CmpGk0


!SUBROUTINE AddSigma(gij, Sigc, DMFTU, sign, iso, nbands, nl, ll, cix, iorbital, norbitals, natom, ncix, maxdim2, lmax2)
!  ! Add's self-energy to inverse of the Green's function in band representation
!  ! 50% of all time is spend in this subroutine, the rest 50 % in CmpGk
!  IMPLICIT NONE
!  COMPLEX*16, intent(inout):: gij(nbands,nbands)
!  COMPLEX*16, intent(in)   :: Sigc(maxdim2,maxdim2,ncix)
!  COMPLEX*16, intent(in)   :: DMFTU(nbands,maxdim2,norbitals)
!  INTEGER, intent(in)      :: sign, iso, nbands, nl(natom), ll(natom,4), cix(natom,4), iorbital(natom,lmax2+1)
!  INTEGER, intent(in)      :: natom, ncix, maxdim2, lmax2, norbitals
!  !------ locals
!  COMPLEX*16,allocatable   :: Up(:,:), tmp(:,:)
!  INTEGER    :: icase, lcase, l1, nind, icix, iorb, ind1, ind2, i, j
!  DO icase=1,natom            ! over correlated blocks: atoms
!     do lcase=1,nl(icase)     !                      and  l's
!        l1=ll(icase,lcase)
!        nind=(2*l1+1)*iso
!        icix = cix(icase,lcase)
!        iorb = iorbital(icase,lcase)
!        if (icix.eq.0) cycle
!        ! g += sign * U * Sig * U^\dagger
!        allocate( Up(nind,nbands), tmp(nbands,nind) )
!        Up = transpose(conjg(DMFTU(:,:nind,iorb)))
!        tmp = matmul(DMFTU(:,:nind,iorb), Sigc(:nind,:nind,icix))
!        gij(:,:) = gij(:,:) + sign*matmul(tmp, Up)
!        deallocate( Up, tmp )
!     enddo
!  ENDDO
!END SUBROUTINE AddSigma

!SUBROUTINE AddSigma_optimized(gij, sigma, STrans, sign, t_size, iso, nbands, nl, ll, cix, iorbital, norbitals, natom, ncix, maxsize, lmax2)
!  !-- Add's self-energy to inverse of the Green's function in band representation
!  !-- 50% of all time is spend in this subroutine, the rest 50 % in CmpGk
!  IMPLICIT NONE
!  COMPLEX*16, intent(inout):: gij(nbands,nbands)
!  COMPLEX*16, intent(in)   :: sigma(maxsize,ncix)
!  COMPLEX*16, intent(out)  :: STrans(maxsize,norbitals,nbands,nbands)
!  INTEGER, intent(in)      :: sign, iso, nbands, nl(natom), ll(natom,4), cix(natom,4), iorbital(natom,lmax2+1), t_size(norbitals)
!  INTEGER, intent(in)      :: natom, ncix, lmax2, norbitals, maxsize
!  !----- locals
!  COMPLEX*16 :: Sigmaij
!  INTEGER    :: icase, lcase, icix, iorb, ind1, ind2, i, j, it
!  
!  DO i=1,nbands                     ! over bands-1
!     do j=1,nbands                  ! over bands-2
!        Sigmaij=0
!        DO icase=1,natom            ! over correlated blocks: atoms
!           do lcase=1,nl(icase)     !                      and  l's
!              icix = cix(icase,lcase)
!              iorb = iorbital(icase,lcase)
!              if (icix.eq.0) cycle
!              do it=1,t_size(iorb)
!                 Sigmaij = Sigmaij + STrans(it,iorb,i,j)*sigma(it,icix)
!              enddo
!           enddo
!        ENDDO
!        gij(i,j) = gij(i,j) + sign*Sigmaij
!     enddo
!  ENDDO
!END SUBROUTINE AddSigma_optimized

!SUBROUTINE CompressSigmaTransformation(STrans, DMFTrans, SindA, noccur, iorbital, t_size, nbands, norbitals, natom, nl, maxsize, maxdim, maxdim2, lmax2)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out)  :: STrans(maxsize,norbitals,nbands,nbands)
!  COMPLEX*16, intent(in)   :: DMFTrans(nbands,nbands,maxdim2,maxdim2,norbitals)
!  INTEGER,    intent(in)   :: SindA(2,maxdim*maxdim,maxsize,norbitals)
!  INTEGER,    intent(in)   :: noccur(maxsize,norbitals), iorbital(natom,lmax2+1), t_size(norbitals), nl(natom)
!  INTEGER,    intent(in)   :: nbands, norbitals, natom, maxsize, maxdim, maxdim2, lmax2
!  !----- locals
!  COMPLEX*16 :: csum
!  INTEGER    :: i, j, icase, lcase, iorb, it, ioccur, ind1, ind2
!  DO i=1,nbands                     ! over bands-1
!     do j=1,nbands                  ! over bands-2
!        DO icase=1,natom            ! over correlated blocks: atoms
!           do lcase=1,nl(icase)     !                      and  l's
!              iorb = iorbital(icase,lcase)
!              do it=1,t_size(iorb)
!                 csum=0
!                 do ioccur=1,noccur(it,iorb)
!                    ind1 = SindA(1, ioccur, it, iorb)
!                    ind2 = SindA(2, ioccur, it, iorb)
!                    csum = csum + DMFTrans(j,i,ind2,ind1,iorb)
!                 enddo
!                 STrans(it,iorb,i,j) = csum
!              enddo
!           enddo
!        ENDDO
!     enddo
!  ENDDO
!END SUBROUTINE CompressSigmaTransformation
!
!
!SUBROUTINE CompressSigmaTransformation1(STrans, DMFTU, SindA, noccur, iorbital, t_size, nbands, norbitals, natom, nl, maxsize, maxdim, maxdim2, lmax2)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out)  :: STrans(maxsize,norbitals,nbands,nbands)
!  COMPLEX*16, intent(in)   :: DMFTU(nbands,maxdim2,norbitals)
!  INTEGER,    intent(in)   :: SindA(2,maxdim*maxdim,maxsize,norbitals)
!  INTEGER,    intent(in)   :: noccur(maxsize,norbitals), iorbital(natom,lmax2+1), t_size(norbitals), nl(natom)
!  INTEGER,    intent(in)   :: nbands, norbitals, natom, maxsize, maxdim, maxdim2, lmax2
!  !----- locals
!  COMPLEX*16 :: csum
!  INTEGER    :: i, j, icase, lcase, iorb, it, ioccur, ind1, ind2
!  DO i=1,nbands                     ! over bands-1
!     do j=1,nbands                  ! over bands-2
!        DO icase=1,natom            ! over correlated blocks: atoms
!           do lcase=1,nl(icase)     !                      and  l's
!              iorb = iorbital(icase,lcase)
!              do it=1,t_size(iorb)
!                 csum=0
!                 do ioccur=1,noccur(it,iorb)
!                    ind1 = SindA(1, ioccur, it, iorb)
!                    ind2 = SindA(2, ioccur, it, iorb)
!                    !csum = csum + DMFTrans(j,i,ind2,ind1,iorb)
!                    csum = csum + conjg(DMFTU(j,ind2,iorb))*DMFTU(i,ind1,iorb)
!                 enddo
!                 STrans(it,iorb,i,j) = csum
!              enddo
!           enddo
!        ENDDO
!     enddo
!  ENDDO
!END SUBROUTINE CompressSigmaTransformation1



!SUBROUTINE CompressSigmaTransformation3(STrans, DMFTrans, Sigind, iSx, cix, csize, iorbital, ll, nl, natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize) 
!  IMPLICIT NONE
!  COMPLEX*16, intent(out) :: STrans(maxsize,ncix,nbands,nbands)
!  COMPLEX*16, intent(in)  :: DMFTrans(nbands,nbands,maxdim2,maxdim2,norbitals)
!  INTEGER, intent(in)     :: Sigind(maxdim,maxdim,ncix), cix(natom,4), csize(ncix)
!  INTEGER, intent(in)     :: iSx(maxdim2, norbitals)
!  INTEGER, intent(in)     :: iorbital(natom,lmax2+1), ll(natom,4), nl(natom)
!  INTEGER, intent(in)     :: natom, iso, ncix, maxdim, maxdim2, lmax2, norbitals, nbands, maxsize
!  !----- locals
!  INTEGER    :: i, j, icase, jcase, lcase, l1case, l2case, iorb, iorb1, iorb2, it, ind1, nind1, ind2, nind2, icix, l1, l2, ip1, ip2
!  STrans=0
!  DO icase=1,natom      
!     do l1case=1,nl(icase) 
!        icix = cix(icase,l1case)
!        if ( icix.EQ.0 ) CYCLE
!        l1 = ll(icase,l1case)
!        nind1 = (2*l1+1)*iso
!        iorb1 = iorbital(icase,l1case)
!        DO jcase=1,natom
!           do l2case=1,nl(jcase)
!              if ( cix(jcase,l2case).NE.icix ) CYCLE
!              l2 = ll(jcase,l2case)
!              nind2 = (2*l2+1)*iso
!              iorb2 = iorbital(jcase,l2case)
!              if (iorb2.NE.iorb1) THEN
!                 print *, 'WARNING: Using MatrixForm=False and multi-atom cix is not allowed. Will make cix diagonal in atom index.'
!                 CYCLE
!              ENDIF
!              
!              if (iorb1.NE.iorb2) print *, 'TEZAVE!!!!????'
!              
!              do ind1=1,nind1
!                 do ind2=1,nind2
!                    it = Sigind( iSx(ind1,iorb1),iSx(ind2,iorb2), icix )
!                    
!                    if (it.gt.0) then
!                       
!                       DO i=1,nbands                     ! over bands-1
!                          do j=1,nbands                  ! over bands-2
!                             STrans(it,icix,i,j) = STrans(it,icix,i,j) + DMFTrans(j,i,ind2,ind1,iorb1)
!                          enddo
!                       ENDDO
!                       
!                    endif
!                 enddo
!              enddo
!           enddo
!        ENDDO
!     enddo
!  ENDDO
!END SUBROUTINE CompressSigmaTransformation3
!
!SUBROUTINE CompressGcTransformation(GTrans, DMFTrans, SindA, noccur, iorbital, t_size, nbands, norbitals, natom, nl, maxsize, maxdim, maxdim2, lmax2)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out)  :: GTrans(nbands,nbands,maxsize,norbitals)
!  COMPLEX*16, intent(in)   :: DMFTrans(nbands,nbands,maxdim2,maxdim2,norbitals)
!  INTEGER,    intent(in)   :: SindA(2,maxdim*maxdim,maxsize,norbitals)
!  INTEGER,    intent(in)   :: noccur(maxsize,norbitals), iorbital(natom,lmax2+1), t_size(norbitals), nl(natom)
!  INTEGER,    intent(in)   :: nbands, norbitals, natom, maxsize, maxdim, maxdim2, lmax2
!  !----- locals
!  COMPLEX*16 :: csum
!  INTEGER    :: i, j, icase, lcase, iorb, it, ioccur, ind1, ind2
!  DO i=1,nbands                     ! over bands-1
!     do j=1,nbands                  ! over bands-2
!        DO icase=1,natom            ! over correlated blocks: atoms
!           do lcase=1,nl(icase)     !                      and  l's
!              iorb = iorbital(icase,lcase)
!              do it=1,t_size(iorb)
!                 csum=0
!                 do ioccur=1,noccur(it,iorb)
!                    ind1 = SindA(1, ioccur, it, iorb)
!                    ind2 = SindA(2, ioccur, it, iorb)
!                    csum = csum + DMFTrans(i,j,ind1,ind2,iorb)
!                 enddo
!                 GTrans(i,j,it,iorb) = csum
!              enddo
!           enddo
!        ENDDO
!     enddo
!  ENDDO
!END SUBROUTINE CompressGcTransformation
!
!
!SUBROUTINE CompressGcTransformation1(GTrans, DMFTrans, DMFTU, SindA, noccur, iorbital, cix, t_size, nbands, norbitals, natom, nl, maxsize, maxdim, maxdim2, lmax2)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out)  :: GTrans(nbands,nbands,maxsize,norbitals)
!  COMPLEX*16, intent(in)   :: DMFTrans(nbands,nbands,maxdim2,maxdim2,norbitals)
!  COMPLEX*16, intent(in)   :: DMFTU(nbands,maxdim2,norbitals)
!  INTEGER,    intent(in)   :: SindA(2,maxdim*maxdim,maxsize,norbitals)
!  INTEGER,    intent(in)   :: noccur(maxsize,norbitals), iorbital(natom,lmax2+1), t_size(norbitals), nl(natom), cix(natom,4)
!  INTEGER,    intent(in)   :: nbands, norbitals, natom, maxsize, maxdim, maxdim2, lmax2
!  !----- locals
!  COMPLEX*16 :: csum
!  INTEGER    :: i, j, icase, lcase, iorb, it, ioccur, ind1, ind2, icix
!  DO i=1,nbands                     ! over bands-1
!     do j=1,nbands                  ! over bands-2
!        DO icase=1,natom            ! over correlated blocks: atoms
!           do lcase=1,nl(icase)     !                      and  l's
!              iorb = iorbital(icase,lcase)
!              icix = cix(icase,lcase)
!              if (icix.gt.0) then
!                 do it=1,t_size(iorb)
!                    csum=0
!                    do ioccur=1,noccur(it,iorb)
!                       ind1 = SindA(1, ioccur, it, iorb)
!                       ind2 = SindA(2, ioccur, it, iorb)
!                       csum = csum + conjg(DMFTU(i,ind1,iorb))*DMFTU(j,ind2,iorb)
!                    enddo
!                    GTrans(i,j,it,iorb) = csum
!                 enddo
!              else
!                 do it=1,t_size(iorb)
!                    csum=0
!                    do ioccur=1,noccur(it,iorb)
!                       ind1 = SindA(1, ioccur, it, iorb)
!                       ind2 = SindA(2, ioccur, it, iorb)
!                       csum = csum + DMFTrans(i,j,ind1,ind2,iorb)
!                    enddo
!                    GTrans(i,j,it,iorb) = csum
!                 enddo
!              endif
!           enddo
!        ENDDO
!     enddo
!  ENDDO
!END SUBROUTINE CompressGcTransformation1
!
!
!SUBROUTINE CompressGcTransformation2(GTrans, DMFTrans, DMFTU, SindA, noccur, iorbital, cix, t_size, nbands, norbitals, natom, nl, maxsize, maxdim, maxdim2, lmax2)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out)  :: GTrans(nbands,nbands,maxsize,norbitals)
!  COMPLEX*16, intent(in)   :: DMFTrans(nbands,nbands,maxdim2,maxdim2,norbitals)
!  COMPLEX*16, intent(in)   :: DMFTU(nbands,maxdim2,norbitals)
!  INTEGER,    intent(in)   :: SindA(2,maxdim*maxdim,maxsize,norbitals)
!  INTEGER,    intent(in)   :: noccur(maxsize,norbitals), iorbital(natom,lmax2+1), t_size(norbitals), nl(natom), cix(natom,4)
!  INTEGER,    intent(in)   :: nbands, norbitals, natom, maxsize, maxdim, maxdim2, lmax2
!  !----- locals
!  COMPLEX*16 :: csum
!  INTEGER    :: i, j, icase, lcase, iorb, it, ioccur, ind1, ind2, icix
!  DO i=1,nbands                     ! over bands-1
!     do j=1,nbands                  ! over bands-2
!        DO icase=1,natom            ! over correlated blocks: atoms
!           do lcase=1,nl(icase)     !                      and  l's
!              iorb = iorbital(icase,lcase)
!              icix = cix(icase,lcase)
!              if (icix.gt.0) then
!                 do it=1,t_size(iorb)
!                    csum=0
!                    do ioccur=1,noccur(it,iorb)
!                       ind1 = SindA(1, ioccur, it, iorb)
!                       ind2 = SindA(2, ioccur, it, iorb)
!                       csum = csum + conjg(DMFTU(i,ind1,iorb))*DMFTU(j,ind2,iorb)
!                    enddo
!                    GTrans(i,j,it,iorb) = csum
!                 enddo
!              else
!                 do it=1,t_size(iorb)
!                    csum=0
!                    do ioccur=1,noccur(it,iorb)
!                       ind1 = SindA(1, ioccur, it, iorb)
!                       ind2 = SindA(2, ioccur, it, iorb)
!                       csum = csum + DMFTrans(i,j,ind1,ind2,iorb)
!                    enddo
!                    GTrans(i,j,it,iorb) = csum
!                 enddo
!              endif
!           enddo
!        ENDDO
!     enddo
!  ENDDO
!END SUBROUTINE CompressGcTransformation2

!SUBROUTINE GetImpurityLevels(Eimp, GTrans, E, EF, noccur, iorbitalcx, csize, nbands, nemin, maxsize, nume, ncix, norbitals)
!  IMPLICIT NONE
!  REAL*8,     intent(out):: Eimp(maxsize,ncix)
!  COMPLEX*16, intent(out):: GTrans(nbands,nbands,maxsize,norbitals)
!  REAL*8,     intent(in) :: E(nume), EF                         ! eigenvalues, chemical potential
!  INTEGER,    intent(in) :: noccur(maxsize,norbitals)
!  INTEGER,    intent(in) :: iorbitalcx(ncix), csize(ncix)
!  INTEGER,    intent(in) :: nbands, nemin, maxsize, nume, ncix, norbitals
!  !----- locals
!  INTEGER    :: it, i, icase, lcase, icix
!  COMPLEX*16 :: wEimp, iorb
!  DO icix=1,ncix             !-------- over correlated blocks: atoms ------------!
!     iorb = iorbitalcx(icix) !--- index to correlated orbit --------------!
!     do it=1,csize(icix)
!        wEimp=0
!        DO i=1,nbands      
!           wEimp = wEimp + (E(i+nemin-1)-EF)*GTrans(i,i,it,iorb) ! Here we NEED to add Sigma_oo-Edc!!!
!        ENDDO
!        Eimp(it,icix) = real(wEimp)/noccur(it,iorb)
!     enddo
!  ENDDO
!END SUBROUTINE GetImpurityLevels
!
!SUBROUTINE GetImpurityLevels1(Eimp, DMFTU, E, EF, iorbitalcx, nindx, nbands, nemin, nume, ncix, norbitals, maxdim2)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out):: Eimp(maxdim2,maxdim2,ncix)
!  COMPLEX*16, intent(out):: DMFTU(nbands,maxdim2,norbitals)
!  REAL*8,     intent(in) :: E(nume), EF                         ! eigenvalues, chemical potential
!  INTEGER,    intent(in) :: iorbitalcx(ncix), nindx(ncix)
!  INTEGER,    intent(in) :: nbands, nemin, nume, ncix, norbitals, maxdim2
!  !----- locals
!  INTEGER    :: it, i, icase, lcase, icix, nind, p
!  COMPLEX*16 :: wEimp, iorb, tmp(maxdim2,nbands)
!  DO icix=1,ncix             !-------- over correlated blocks: atoms ------------!
!     iorb = iorbitalcx(icix) !--- index to correlated orbit --------------!
!     nind = nindx(icix)
!     do p=1,nind
!        do i=1,nbands
!           tmp(p,i) = conjg(DMFTU(i,p,iorb))*(E(i+nemin-1)-EF) ! Here we NEED to add Sigma_oo-Edc!!!
!        enddo
!     enddo
!     Eimp(:nind,:nind,icix) = matmul(tmp(:nind,:), DMFTU(:,:nind,iorb))
!  ENDDO
!END SUBROUTINE GetImpurityLevels1

!SUBROUTINE GetOverlap(Olap, GTrans, noccur, iorbitalcx, csize, nbands, nemin, maxsize, nume, ncix, norbitals)
!  IMPLICIT NONE
!  REAL*8,     intent(out):: Olap(maxsize,ncix)
!  COMPLEX*16, intent(out):: GTrans(nbands,nbands,maxsize,norbitals)
!  INTEGER,    intent(in) :: noccur(maxsize,norbitals)
!  INTEGER,    intent(in) :: iorbitalcx(ncix), csize(ncix)
!  INTEGER,    intent(in) :: nbands, nemin, maxsize, nume, ncix, norbitals
!  !----- locals
!  INTEGER    :: it, i, icase, lcase, icix
!  COMPLEX*16 :: wOlap, iorb
!  DO icix=1,ncix             !-------- over correlated blocks: atoms ------------!
!     iorb = iorbitalcx(icix) !--- index to correlated orbit --------------!
!     do it=1,csize(icix)
!        wOlap=0
!        DO i=1,nbands      
!           wOlap = wOlap + GTrans(i,i,it,iorb)
!        ENDDO
!        Olap(it,icix) = real(wOlap)/noccur(it,iorb)
!     enddo
!  ENDDO
!END SUBROUTINE GetOverlap
!
!SUBROUTINE GetOverlap1(Olap, DMFTU, iorbitalcx, nindx, nbands, maxdim2, ncix, norbitals)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out):: Olap(maxdim2,maxdim2,ncix)
!  COMPLEX*16, intent(in) :: DMFTU(nbands,maxdim2,norbitals)
!  INTEGER,    intent(in) :: iorbitalcx(ncix), nindx(ncix)
!  INTEGER,    intent(in) :: nbands, maxdim2, ncix, norbitals
!  !----- locals
!  INTEGER    :: nind, iorb, icix
!
!  DO icix=1,ncix             !-------- over correlated blocks: atoms ------------!
!     iorb = iorbitalcx(icix) !--- index to correlated orbit --------------!
!     nind = nindx(icix)
!     Olap(:nind,:nind,icix) = matmul(transpose(conjg(DMFTU(:,:nind,iorb))), DMFTU(:,:nind,iorb))
!  ENDDO
!END SUBROUTINE GetOverlap1


!SUBROUTINE CmpGk(gk, gij, DMFTU, GTrans, t_size, icix, ncix, noccur, nind, nbands, maxdim2, maxsize)
!  ! DMFTrans(i,j,ip,iq,iorb)-conjg(DMFTU(i,ip,iorb))*DMFTU(j,iq,iorb)).gt.1e-4) then
!  ! Creates local green's function
!  ! 50% of all time is spend in this subroutine, the rest 50 % in AddSigma
!  IMPLICIT NONE
!  COMPLEX*16, intent(out):: gk(maxdim2,maxdim2)
!  COMPLEX*16, intent(in) :: gij(nbands,nbands)
!  COMPLEX*16, intent(in) :: DMFTU(nbands,maxdim2)
!  COMPLEX*16, intent(out):: GTrans(nbands,nbands,maxsize)
!  INTEGER, intent(in) :: nind, icix, ncix, nbands, maxdim2, t_size, maxsize, noccur(maxsize)
!  ! locals
!  COMPLEX*16, allocatable :: Up(:,:), tmp(:,:)
!  INTEGER    :: it, i, j
!  COMPLEX*16 :: gc
!  if (icix.gt.0) then
!     allocate( tmp(nind,nbands) )
!     
!     !allocate( Up(nind,nbands) )
!     !Up = transpose(conjg(DMFTU(:,:nind)))
!     !tmp = matmul(Up, gij)
!     !gk(:nind,:nind) = matmul(tmp, DMFTU(:,:nind))
!
!     
!     call zgemm('C','N', nind, nbands, nbands, (1.d0,0.d0), DMFTU,nbands, gij,nbands, 0.0, tmp,nind)
!     call zgemm('N','N', nind,  nind,  nbands, (1.d0,0.d0), tmp,nind, DMFTU,nbands, 0.0, gk,maxdim2)
!
!     !deallocate( Up )
!
!     deallocate( tmp )
!  else
!     
!     if (ncix.gt.0) then
!        do it=1,t_size
!           gc=0
!           DO i=1,nbands      
!              DO j=1,nbands
!                 gc = gc + gij(i,j)*GTrans(i,j,it)
!              ENDDO
!           ENDDO
!           gk(it,1)=gc/noccur(it)
!        enddo
!     else
!        do it=1,t_size
!           gc=0
!           DO i=1,nbands      
!              gc = gc + gij(i,i)*GTrans(i,i,it)
!           ENDDO
!           gk(it,1)=gc/noccur(it)
!        enddo
!     endif
!
!  endif
!END SUBROUTINE CmpGk
!
!SUBROUTINE CmpGk_optimized(gk, gij, GTrans, t_size, noccur, ncix, nbands, maxsize)
!  ! Creates local green's function
!  ! 50% of all time is spend in this subroutine, the rest 50 % in AddSigma
!  IMPLICIT NONE
!  COMPLEX*16, intent(out):: gk(maxsize)
!  COMPLEX*16, intent(in) :: gij(nbands,nbands)
!  COMPLEX*16, intent(out):: GTrans(nbands,nbands,maxsize)
!  INTEGER,    intent(in) :: t_size, noccur(maxsize)
!  INTEGER, intent(in)    :: ncix, nbands, maxsize
!  !------ locals
!  INTEGER    :: it, i, j
!  COMPLEX*16 :: gc
!  do it=1,t_size
!     gc=0
!     if (ncix.gt.0) then    ! off-diagonal g in band representation
!        DO i=1,nbands                      ! over bands-1
!           do j=1,nbands                   ! over bands-2
!              gc = gc + gij(i,j)*GTrans(i,j,it)
!           enddo
!        ENDDO
!     else ! for non-correlated case, green's function is diagonal in band representation
!        DO i=1,nbands      
!           gc = gc + gij(i,i)*GTrans(i,i,it)
!        ENDDO
!     endif
!     gk(it)=gc/noccur(it)
!  enddo
!END SUBROUTINE CmpGk_optimized



!SUBROUTINE GetDelta(Deltac, Eimpc, matsubara, gloc, sigma, omega, Eimp, Olap, Sigind, csize, nl, ll, cix, iorbital, noccur, iso, nbands, natom, ncix, lmax2, norbitals, maxsize, maxdim, nomega)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out):: Deltac(maxsize, nomega, ncix)
!  REAL*8, intent(out)    :: Eimpc(maxsize, ncix)
!  LOGICAL, intent(in)    :: matsubara
!  COMPLEX*16, intent(in) :: gloc(maxsize,nomega,norbitals), sigma(maxsize,ncix,nomega)
!  REAL*8,  intent(in)    :: omega(nomega), Eimp(maxsize,ncix), Olap(maxsize,ncix)
!  INTEGER, intent(in)    :: Sigind(maxdim,maxdim,ncix), csize(ncix)
!  INTEGER, intent(in)    :: nl(natom), ll(natom,4), cix(natom,4), iorbital(natom,lmax2+1), noccur(maxsize,norbitals)
!  INTEGER, intent(in)    :: iso, nbands, natom, ncix, lmax2, norbitals, maxsize, nomega, maxdim
!  !----- locals
!  COMPLEX*16, allocatable:: gcx(:,:), Eimpx(:,:), deltx(:,:), Olapx(:,:), sigcx(:,:) 
!  COMPLEX*16 :: xomega
!  INTEGER    :: icase, lcase, l1, icix, nind, iorb, ip, iq, it, iom, isi
!  REAL*8     :: dsum
!  complex*16 :: IMAG
!  !-----------------------
!  DATA IMAG/(0.0D0,1.0D0)/
!  !-----------------------        
!  Deltac = 0
!  DO icase=1,natom
!     do lcase=1,nl(icase)
!        l1 = ll(icase,lcase)
!        nind = (2*l1+1)*iso
!        icix = cix(icase,lcase)
!        iorb = iorbital(icase,lcase)
!        if (icix.eq.0) cycle
!
!        allocate( gcx(nind,nind), Eimpx(nind,nind), deltx(nind,nind), sigcx(nind,nind), Olapx(nind,nind) )
!        !---- unpacking Eimp to matrix form
!        !Idx=0    !--- identity matrix
!        Eimpx=0  !--- impurity levels
!        Olapx=0
!        DO ip=1,nind
!           do iq=1,nind
!              if (Sigind(ip,iq,icix).gt.0) then
!                 Eimpx(ip,iq) = Eimp( Sigind(ip,iq,icix), icix )
!                 Olapx(ip,iq) = Olap( Sigind(ip,iq,icix), icix )
!              endif
!           enddo
!        ENDDO
!
!        CALL zinv(Olapx, nind)
!        
!        Eimpx = matmul( matmul(Olapx,Eimpx), Olapx) !-- normalize by overlap
!        !DO ip=1,ndim
!        !   Idx(ip,ip) = 1/Olap(ip,ip) !-- This will make sure that high frequency is correct
!        !ENDDO
!
!        !---- Eimpc to vector form
!        DO ip=1,nind
!           do iq=1,nind
!              isi = Sigind(ip,iq,icix)
!              if (isi.gt.0) then
!                 Eimpc(isi, icix) =  Eimpc(isi, icix) + Eimpx(ip,iq)
!              endif
!           enddo
!        ENDDO
!        DO it=1,csize(icix)
!           Eimpc(it, icix) = Eimpc(it, icix)/noccur(it,iorb)
!        ENDDO
!
!        
!        do iom=1,nomega !------- for all frequencies --!
!           
!           IF (matsubara) THEN
!              xomega = omega(iom)*IMAG
!           ELSE
!              xomega = omega(iom)
!           ENDIF
!           !---- unpacking gloc to matrix form
!           gcx=0
!           sigcx=0
!           DO ip=1,nind
!              do iq=1,nind
!                 if (Sigind(ip,iq,icix).gt.0) then
!                    gcx  (ip,iq) = gloc ( Sigind(ip,iq,icix), iom, iorb )
!                    sigcx(iq,ip) = sigma( Sigind(ip,iq,icix), icix, iom )
!                 endif
!              enddo
!           ENDDO
!           
!           CALL zinv(gcx,nind)
!           
!           !--- DMFT self-conistency condition
!           DO ip=1,nind
!              do iq=1,nind
!                 deltx(ip,iq) = Olapx(ip,iq)*xomega - Eimpx(ip,iq) - gcx(ip,iq) - sigcx(ip,iq)
!              enddo
!           ENDDO
!           
!           !---- packing delta to vector form
!           DO ip=1,nind
!              do iq=1,nind
!                 isi = Sigind(ip,iq,icix)
!                 if (isi.gt.0) then
!                    Deltac( isi, iom, icix) =  Deltac( isi, iom, icix) + deltx(ip,iq)
!                 endif
!              enddo
!           ENDDO
!           DO it=1,csize(icix)
!              Deltac(it, iom, icix) = Deltac(it, iom, icix)/noccur(it,iorb)
!           ENDDO
!
!        enddo
!
!        deallocate( gcx, Eimpx, deltx, Olapx, sigcx )
!        
!     enddo
!  ENDDO
!END SUBROUTINE GetDelta
!
!SUBROUTINE GetDelta0(Deltac, Eimpc, matsubara, gloc, sigma, omega, Eimp, Olap, Sigind, csize, nindx, ncix, iorbitalcx, noccur, norbitals, maxsize, maxdim, nomega)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out):: Deltac(maxsize, nomega, ncix)
!  REAL*8, intent(out)    :: Eimpc(maxsize, ncix)
!  LOGICAL, intent(in)    :: matsubara
!  COMPLEX*16, intent(in) :: gloc(maxsize,nomega,norbitals), sigma(maxsize,ncix,nomega)
!  REAL*8,  intent(in)    :: omega(nomega), Eimp(maxsize,ncix), Olap(maxsize,ncix)
!  INTEGER, intent(in)    :: Sigind(maxdim,maxdim,ncix), csize(ncix), nindx(ncix)
!  INTEGER, intent(in)    :: ncix, iorbitalcx(ncix), noccur(maxsize,norbitals)
!  INTEGER, intent(in)    :: norbitals, maxsize, nomega, maxdim
!  !----- locals
!  COMPLEX*16, allocatable:: gcx(:,:), Eimpx(:,:), deltx(:,:), Olapx(:,:), sigcx(:,:) 
!  COMPLEX*16 :: xomega
!  INTEGER    :: icase, lcase, l1, icix, nind, iorb, ip, iq, it, iom, isi
!  REAL*8     :: dsum
!  complex*16 :: IMAG
!  !-----------------------
!  DATA IMAG/(0.0D0,1.0D0)/
!  !-----------------------
!  Deltac = 0
!  DO icix=1,ncix
!     nind = nindx(icix)
!     iorb = iorbitalcx(icix)
!     allocate( gcx(nind,nind), Eimpx(nind,nind), deltx(nind,nind), sigcx(nind,nind), Olapx(nind,nind) )
!     !---- unpacking Eimp to matrix form
!     Eimpx=0  !--- impurity levels
!     Olapx=0
!     DO ip=1,nind
!        do iq=1,nind
!           if (Sigind(ip,iq,icix).gt.0) then
!              Eimpx(ip,iq) = Eimp( Sigind(ip,iq,icix), icix )
!              Olapx(ip,iq) = Olap( Sigind(ip,iq,icix), icix )
!           endif
!        enddo
!     ENDDO
!
!     CALL zinv(Olapx, nind)
!        
!     Eimpx = matmul( matmul(Olapx,Eimpx), Olapx) !-- normalize by overlap
!     
!     !---- Eimpc to vector form
!     DO ip=1,nind
!        do iq=1,nind
!           isi = Sigind(ip,iq,icix)
!           if (isi.gt.0) then
!              Eimpc(isi, icix) =  Eimpc(isi, icix) + Eimpx(ip,iq)
!           endif
!        enddo
!     ENDDO
!     DO it=1,csize(icix)
!        Eimpc(it, icix) = Eimpc(it, icix)/noccur(it,iorb)
!     ENDDO
!
!        
!     do iom=1,nomega !------- for all frequencies --!
!           
!        IF (matsubara) THEN
!           xomega = omega(iom)*IMAG
!        ELSE
!           xomega = omega(iom)
!        ENDIF
!        !---- unpacking gloc to matrix form
!        gcx=0
!        sigcx=0
!        DO ip=1,nind
!           do iq=1,nind
!              if (Sigind(ip,iq,icix).gt.0) then
!                 gcx  (ip,iq) = gloc ( Sigind(ip,iq,icix), iom, iorb )
!                 sigcx(iq,ip) = sigma( Sigind(ip,iq,icix), icix, iom )
!              endif
!           enddo
!        ENDDO
!           
!        CALL zinv(gcx,nind)
!           
!        !--- DMFT self-conistency condition
!        DO ip=1,nind
!           do iq=1,nind
!              deltx(ip,iq) = Olapx(ip,iq)*xomega - Eimpx(ip,iq) - gcx(ip,iq) - sigcx(ip,iq)
!           enddo
!        ENDDO
!           
!        !---- packing delta to vector form
!        DO ip=1,nind
!           do iq=1,nind
!              isi = Sigind(ip,iq,icix)
!              if (isi.gt.0) then
!                 Deltac( isi, iom, icix) =  Deltac( isi, iom, icix) + deltx(ip,iq)
!              endif
!           enddo
!        ENDDO
!        DO it=1,csize(icix)
!           Deltac(it, iom, icix) = Deltac(it, iom, icix)/noccur(it,iorb)
!        ENDDO
!
!     enddo
!     
!     deallocate( gcx, Eimpx, deltx, Olapx, sigcx )
!  ENDDO
!END SUBROUTINE GetDelta0
!
!SUBROUTINE GetDelta1(Deltac, Eimpc, matsubara, omega, sigma, gloc, Eimpm, Olapm, Sigind, csize, iorbitalcx, noccur, nindx, ncix, norbitals, maxsize, maxdim, maxdim2, nomega)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out):: Deltac(maxsize, nomega, ncix)
!  REAL*8, intent(out)    :: Eimpc(maxsize, ncix)
!  LOGICAL, intent(in)    :: matsubara
!  REAL*8,  intent(in)    :: omega(nomega)
!  COMPLEX*16, intent(in) :: sigma(maxsize,ncix,nomega), gloc(maxdim2,maxdim2,nomega,norbitals)
!  COMPLEX*16, intent(in) :: Eimpm(maxdim2,maxdim2,ncix), Olapm(maxdim2,maxdim2,ncix)
!  INTEGER, intent(in)    :: Sigind(maxdim,maxdim,ncix), csize(ncix)
!  INTEGER, intent(in)    :: iorbitalcx(ncix), noccur(maxsize,norbitals), nindx(ncix)
!  INTEGER, intent(in)    :: ncix, norbitals, maxsize, maxdim, maxdim2, nomega
!  !----- locals
!  COMPLEX*16, allocatable:: gcx(:,:), Eimpx(:,:), deltx(:,:), sigcx(:,:) 
!  COMPLEX*16 :: xomega
!  INTEGER    :: icix, nind, iorb, ip, iq, it, iom, isi
!  complex*16 :: IMAG
!  !-----------------------
!  DATA IMAG/(0.0D0,1.0D0)/
!  !-----------------------
!  Deltac = 0
!  DO icix=1,ncix
!     nind = nindx(icix)
!     iorb = iorbitalcx(icix)
!     allocate( Eimpx(nind,nind), deltx(nind,nind), sigcx(nind,nind), gcx(nind,nind) )
!     
!
!     !print *, 'before first zinv'
!     !print *, 'nind=', nind
!     !print *, 'Olapm=', Olapm(:nind,:nind,icix)
!
!     ! O^{-1} needed to Eimp = O^{-1} H_{loc} O^{-1}
!     CALL zinv(Olapm(:nind,:nind,icix), nind)
!     Eimpx = matmul( matmul(Olapm(:nind,:nind,icix),Eimpm(:nind,:nind,icix)), Olapm(:nind,:nind,icix)) !-- normalize by overlap
!     
!     !---- Eimpc to vector form
!     DO ip=1,nind
!        do iq=1,nind
!           isi = Sigind(ip,iq,icix)
!           if (isi.gt.0) then
!              Eimpc(isi, icix) =  Eimpc(isi, icix) + Eimpx(ip,iq)
!           endif
!        enddo
!     ENDDO
!     DO it=1,csize(icix)
!        Eimpc(it, icix) = Eimpc(it, icix)/noccur(it,iorb)
!     ENDDO
!     
!     do iom=1,nomega !------- for all frequencies --!
!           
!        IF (matsubara) THEN
!           xomega = omega(iom)*IMAG
!        ELSE
!           xomega = omega(iom)
!        ENDIF
!        !---- unpacking sigma to matrix form
!        sigcx=0
!        DO ip=1,nind
!           do iq=1,nind
!              if (Sigind(ip,iq,icix).gt.0) then
!                 sigcx(iq,ip) = sigma( Sigind(ip,iq,icix), icix, iom )
!              endif
!           enddo
!        ENDDO
!        
!        gcx = gloc(:nind,:nind,iom,iorb)
!        CALL zinv(gcx,nind)
!        
!        !--- DMFT self-conistency condition
!        deltx = Olapm(:nind,:nind,icix)*xomega - Eimpx - gcx - sigcx
!        
!        !---- packing delta to vector form
!        DO ip=1,nind
!           do iq=1,nind
!              isi = Sigind(ip,iq,icix)
!              if (isi.gt.0) then
!                 Deltac( isi, iom, icix) =  Deltac( isi, iom, icix) + deltx(ip,iq)
!              endif
!           enddo
!        ENDDO
!        DO it=1,csize(icix)
!           Deltac(it, iom, icix) = Deltac(it, iom, icix)/noccur(it,iorb)
!        ENDDO
!
!     enddo
!     
!     deallocate( Eimpx, deltx, sigcx, gcx )
!  ENDDO
!END SUBROUTINE GetDelta1


!SUBROUTINE SetSigc(Sigc, sigma, Sigind, nl, ll, cix, iso, natom, nomega, ncix, maxdim, maxdim2, maxsize)
!  IMPLICIT NONE
!  COMPLEX*16, intent(out) :: Sigc(maxdim2,maxdim2,ncix,nomega) !-- self-energy in matrix form --!
!  COMPLEX*16, intent(in)  :: sigma(maxsize,ncix,nomega)
!  INTEGER, intent(in)     :: Sigind(maxdim,maxdim,ncix)
!  INTEGER, intent(in)     :: nl(natom), ll(natom,4), cix(natom,4)
!  INTEGER, intent(in)     :: nomega, ncix, maxdim, maxdim2, maxsize, natom, iso
!  !---- locals
!  INTEGER :: icase, lcase, icix, l1, nind, ind1, ind2, iom
!  !------------ Input self-energy is in a form of vector with dimension ncix -----------!
!  !------------ We build matrix in orbital base using the correlated index Sigind ------!
!
!  Sigc=0
!  DO icase=1,natom           !--- all atoms requested in the input ----!
!     do lcase=1,nl(icase)    !--- all l's requested in the input ------!
!        l1=ll(icase,lcase)
!        nind=(2*l1+1)*iso
!        icix = cix(icase,lcase)
!        if (icix.eq.0) cycle !--- only correlated orbitals need Sigc --!
!        do ind1=1,nind
!           do ind2=1,nind
!              if (Sigind(ind1,ind2,icix).gt.0) then
!                 do iom=1,nomega
!                    Sigc(ind1,ind2,icix,iom) = sigma(Sigind(ind1,ind2,icix),icix,iom)
!                 enddo
!              endif
!           enddo
!        enddo
!     enddo
!  ENDDO
!END SUBROUTINE SetSigc


SUBROUTINE RenormalizeTrans(DMFTU, Olapm0, cix_orb, nindo, iSx, nbands, maxdim2, norbitals, maxdim, ncix)
  IMPLICIT NONE
  COMPLEX*16, intent(inout) :: DMFTU(nbands,maxdim2,norbitals)
  COMPLEX*16, intent(in)    :: Olapm0(maxdim,maxdim,ncix)
  INTEGER, intent(in)       :: cix_orb(norbitals), nindo(norbitals), iSx(maxdim2,norbitals)
  INTEGER, intent(in)       :: nbands, maxdim2, norbitals, maxdim, ncix
  ! locals
  INTEGER :: iorb1, icix, nind1, ind1
  REAL*8  :: olocef

  DO iorb1=1,norbitals
     icix = cix_orb(iorb1)
     if ( icix.EQ.0 ) CYCLE
     nind1 = nindo(iorb1)
     
     do ind1=1,nind1
        olocef = 1/sqrt(real(Olapm0( iSx(ind1,iorb1), iSx(ind1,iorb1), icix )))
        DMFTU(:,ind1,iorb1) = DMFTU(:,ind1,iorb1) * olocef
     enddo
     
  ENDDO
END SUBROUTINE RenormalizeTrans
