from scipy import *
import struct, sys, re
import rdU


class Struct:
    def __init__(self, case):
        self.case = case
        self.extn = 'struct'
        self.parse()

    def parse(self):
        '''The file is structured for reading by fortran code,
        so data is positioned by line and by column.'''
        f = open(self.case + '.' + self.extn, 'r')
        self.title = f.next().strip()

        line = f.next()
        self.lattice = line[0:4].strip()
        self.nat = int(line[27:30])
        self.latticename = line[30:].strip()

        self.mode = f.next()[13:17]

        line = f.next()
        self.a, self.b, self.c, self.alpha, self.beta, self.gamma = [float(line[i:i+10]) for i in range(0,60,10)]

        self.iatom  = []
        self.mult   = []
        self.isplit = []
        self.aname  = []
        self.npt    = []
        self.r0     = []
        self.rmt    = []
        self.Znuc   = []
        self.pos    = []
        self.rotloc = []

        for iat in range(self.nat):
            line = f.next()
            self.iatom.append( int(line[4:8]) )
            pos = [[float(line[col:col+10]) for col in 12+13*arange(3)]]
            
            line = f.next()
            mult = int(line[15:17])
            self.mult.append(mult)

            self.isplit.append( int(line[34:37]) )

            for mu in range(mult-1):
                line = f.next()
                pos.append( [float(line[col:col+10]) for col in 12+13*arange(3)] )

            self.pos.append(pos)
                
            line = f.next()
            self.aname.append( line[0:10].strip() )
            self.npt.append( int(line[15:20]) )
            self.r0.append( float(line[25:35]) )
            self.rmt.append( float(line[40:50]) )
            self.Znuc.append( int(float(line[55:65])) )

            rt=[]
            for i in range(3):
                line = f.next()
                rt.append( [float(line[col:col+10]) for col in 20+10*arange(3)] )
            self.rotloc.append(rt)

        self.Ng = int(f.next().split()[0])
        self.iz=[]
        self.tau=[]
        for i in range(self.Ng):
            tiz=zeros((3,3), dtype=float)
            ttau=zeros(3, dtype=float)
            for j in range(3):
                line = f.next()
                tiz[j,:] = array([int(line[0:2]), int(line[2:4]), int(line[4:6])])
                ttau[j] = float(line[6:16])
            self.iz.append(tiz)
            self.tau.append(ttau)
            f.next()
        #print self.iz
        
        f.close()

    def debugprint(self, f=sys.stdout):
        print >> f, '***** Structure File Contents *****'
        print >> f, 'title =', self.title
        print >> f, 'Number of sorts, nat =', self.nat
        print >> f, 'unit cell parameters (a,b,c) =', self.a, self.b, self.c
        print >> f, 'angles (alpha, beta, gamma) =', self.alpha, self.beta, self.gamma


        printlist = [
            (self.aname,  'Atom name, aname ='),
            (self.iatom,  'Atom index, iatom ='),
            (self.mult,   'Number of atoms of this type, mult ='),
            (self.isplit, 'Symmetry of the atom, isplit ='),
            (self.npt,    'Number of radial points, npt ='),
            (self.r0,     'First point in radial mesh, r0 ='),
            (self.rmt,    'Muffin tin radius, rmt ='),
            (self.Znuc,   'Atomic number, Znuc ='),
            ]

        for i in range(self.nat):
            print >> f, '---------- atom type', i, '------------'

            for var,docstr in printlist:
                print >> f, docstr, var[i]

            print >> f, 'Position(s) inside the unit cell:'
            for m in range(self.mult[i]):
                print >> f, '    pos =', self.pos[i][m]

            print >> f, 'Local rotation matrix:'
            for row in self.rotloc[i]:
                print >> f, row

        print >> f, '***********************************'

    def flat(self, notflat):
        '''Return a flat view of given data as a list.
        Example: if w.mult = [2,4] and w.aname = ['V', 'O']
        w.flatten(w.aname) -> ['V', 'V', 'O', 'O', 'O', 'O']'''
        if notflat is self.pos:
            listoflists = self.pos
        else:
            listoflists = [[elem]*mult for elem,mult in zip(notflat, self.mult)]
        return reduce(operator.add, listoflists)


def DirectVectors(BR,RN):
    v0 = cross(BR[1],BR[2])
    v1 = cross(BR[2],BR[0])
    v2 = cross(BR[0],BR[1])
    vol = dot(v0,BR[0])
    return dot(RN, array([v0/vol,v1/vol,v2/vol]))
    
def Print(R, tHr):
    print 'Hop[',tuple(R),']=array(',
    #for l1 in range(shape(tHr)[0]):
    #    for l2 in range(shape(tHr)[1]):
    #        #if abs(tHr[l1,l2])<1e-10:
    #        #    tHr[l1,l1]=0.0
    #        #elif abs(tHr[l1,l2].imag)<1e-10:
    #        #    tHr[l1,l2]=tHr[l1,l2].real+0j
    #        #elif abs(tHr[l1,l2].real)<1e-10:
    #        #    tHr[l1,l2]=tHr[l1,l2].imag*1j
    print tHr.tolist(),
    print ')'

fhp = 233
fhe=60
cpuID = 0
case = 'BFA'
Ry2eV = 13.6058
EF = 7.908507308
bnd_s = 6
bnd_e = 18


strc = Struct(case)
strc.parse()

# reads BR1 and BR2 from case.rotlm
fr = open(case+'.rotlm') 
BR = zeros((2,3,3), dtype=float)
for ii in range(2):
    fr.next()
    for i in range(3):
        BR[ii,:,i] = map(float,fr.next().split())

BRX = array(linalg.inv(BR[1]) * matrix(BR[0]))

# Computes direct vectors, knowing the reciprocal basis
DV = DirectVectors(BR[1],BR[0])

# Reads div from case.klist
firstline = open(case+'.klist').next()
m = re.search('div: *\((\s*\d\s*\d\s*\d)\)', firstline)
if m is not None:
    div = map(int,m.groups()[0].split())
#print div

# Read Udmft
filename = 'Udmft.'+str(cpuID)
(nkp, nsymop, norbitals) = rdU.fopen(fhp, filename)
nindo = rdU.read1(fhp, norbitals)

#print 'nat=', strc.nat

rdU.feopen(fhe, case+'.energy', strc.nat)

Hk=[]
kps=[]
wgh=[]
wgh0=[]
for ikp in range(nkp):

    (iikp, nbands, tmaxdim2, tnorbitals, nemin) = rdU.read2(fhp)
    (k, kname, wg, ios, nen) = rdU.feread1(fhe)

    if ios!=0:
        print 'Number of k-points in case.energy is smaller then in Udfmt.0'
        break
    Ek = rdU.feread2(fhe, nen) * Ry2eV - EF
    
    wgh0.append(wg)
    
    #k_car = [sum(BR[0][i,:]*k) for i in range(3)]    
    
    for isym in range(nsymop):

        kn = dot(strc.iz[isym],k)

        k_nat = [sum(BRX[i,:]*kn) for i in range(3)]

        #print ikp, isym, k_nat
        
        kps.append(kn)
        wgh.append(wg)
        
        iisym = rdU.read3(fhp)
        if iisym!=isym+1:
            print 'ERROR: isym!=iisym', isym, iisym

        wRt=[]
        for iorb in range(norbitals):

            U = []
            for ind in range(nindo[iorb]):
                U.append( rdU.read4(fhp, nbands)[bnd_s:bnd_e] )
            U = array(U)
            
            (u, s, v) = linalg.svd(U)
            
            Rt = zeros((len(u),len(v)), dtype=complex)
            for p in range(len(u)):
                for i in range(len(v)):
                    dsum=0.
                    for q in range(len(s)): dsum += u[p,q]*v[q,i]
                    Rt[p,i] = dsum
            wRt.append( Rt )
            Nd = len(u)
            
        wHk = zeros((norbitals*Nd,norbitals*Nd),dtype=float)
        for iorb1 in range(norbitals):
            N1 = shape(wRt[iorb1])[0]
            for iorb2 in range(norbitals):
                N2 = shape(wRt[iorb2])[0]
                tHk = dot(conjugate(wRt[iorb1]) * Ek[nemin+bnd_s:nemin+bnd_e], transpose(wRt[iorb2]))
                wHk[iorb1*Nd:(iorb1+1)*Nd,iorb2*Nd:(iorb2+1)*Nd] = tHk
        #print 'e1=', Ek[nemin+bnd_s:nemin+bnd_e]
        #print 'e2=', linalg.eigvalsh(wHk)
        
        Hk.append(wHk)
        
    #kps.append(kn)
    #wgh.append(wg)
    #Hk.append([ikp])


rdU.fclose(fhp)
rdU.feclose(fhe)
            
Hk = array(Hk)     


nrm = sum(wgh0)

Rs=[]

for i0 in range(div[0]/2,-div[0]/2,-1):
    for i1 in range(div[1]/2,-div[1]/2,-1):
        for i2 in range(div[2]/2,-div[2]/2,-1):
            Rs.append(DV[0]*i0 + DV[1]*i1 + DV[2]*i2)

            
Hr=[]
for R in Rs:
    tHr=zeros(shape(Hk[0]),dtype=complex)
    for ik in range(len(kps)):
        expk = exp(1j*2*pi*dot(kps[ik],R)) * wgh[ik] #/(strc.Ng*nrm)
        tHr += expk * Hk[ik]
    Hr.append(tHr)

#print 'R='
#for R in Rs:
#    print tuple(R), ',',

#for ik in range(len(kps)):
#    H0 = zeros(shape(Hr[0]), dtype=complex)
#    for ir,R in enumerate(Rs):
#        expk = exp(-1j*2*pi*dot(kps[ik],R))/len(Rs)
#        H0 += Hr[ir]*expk
#    print ik, H0
#    print ik, Hk[ik]


print 'Hop={}'
for ir,R in enumerate(Rs):
    Print(R, Hr[ir])
    print
    
