#!/usr/bin/env python
import utils,indmffile,sys,re,os
import optparse
from scipy import *

import numpy
if numpy.__version__ == '1.0.1':
    loadtxt = io.read_array
    def savetxt(filename, data):
        io.write_array(filename, data, precision=16)  

def SimplifySiginds(siginds):
    " Takes dictionary of Sigind's and creates list or non-zero columns"

    def union(data):
        " Takes a union of array or list"
        c = []
        for d in data:
            if d not in c:
                c.append(d)
        return c
    
    cols={}
    for icix in siginds.keys():
        Sigind = siginds[icix]
        col = sort(union(array(Sigind).flatten())).tolist()
        if 0 in col: col.remove(0)
        cols[icix] = col
    return cols


if __name__=='__main__':
    """ Takes the self-energy files from all impurity problems, and combines
    them into a single self-energy file.
    """
    usage = """usage: %prog [ options ]

    The script takes the self-energy files from all impurity problems,
    and combines them into a single self-energy file.

    To give filename, you can use the following expressions:
      - word 'case', which will be replaced by current case, determined by
                    the presence of struct file.
      - '?', which will be replaced by icix.
    """

    parser = optparse.OptionParser(usage)
    parser.add_option("-o", "--osig", dest="osig", default='sig.inp', help="filename of the output self-energy file. Default: 'sig.inp'")
    parser.add_option("-i", "--isig", dest="isig", default='imp.?/sig.out', help="filename of the input self-energy from all impurity problems. Default: 'imp.?/sig.out'")
    parser.add_option("-l", "--lext", dest="m_extn", default='', help="For magnetic calculation, it can be 'dn'.")
    
    # Next, parse the arguments
    (options, args) = parser.parse_args()
    
    env = utils.W2kEnvironment()
    case = env.case
    
    options.osig = re.sub(r'case', case, options.osig)
    options.isig = re.sub(r'case', case, options.isig)
    
    print 'case=%s, isig=%s, osig=%s' %  (case, options.isig, options.osig)
    
    
    inl = indmffile.Indmfl(case)
    inl.read()
    if options.m_extn:
        inldn = indmffile.Indmfl(case, 'indmfl'+options.m_extn)
        inldn.read()
                         
    iSiginds = utils.ParsIndmfi(case)
    icols = SimplifySiginds(iSiginds)
    print 'icols=', icols
    
    cols = SimplifySiginds(inl.siginds)
    print 'cols=', cols

    if options.m_extn:
        colsdn = SimplifySiginds(inldn.siginds)
        print 'colsdn=', colsdn


    allcols = array(icols.values()).flatten()
    noccur = zeros(max(allcols),dtype=int)
    for c in allcols:
        noccur[c-1]+=1

    filename = re.sub(r'\?', str(0), options.isig)
    om = loadtxt(filename).transpose()[0]
    
    # Array of all sigmas
    rSigma=zeros(( max(allcols)*2+1, len(om) ),dtype=float)
    rs_oo=zeros( max(allcols) ,dtype=float)
    rEdc=zeros( max(allcols) ,dtype=float)
    rSigma[0] = om

    # Reading self-energies from all impurity problems
    for icix in icols.keys():
        # Processing Delta
        mincol = min(icols[icix])
        maxcol = max(icols[icix])
        filename = re.sub(r'\?', str(icix), options.isig)
        print ('icix=%d reading from: %s' % (icix, filename)), 'cols=', icols[icix]
        
        # Searching for s_oo and Edc
        fh_sig = open(filename, 'r')
        exec(fh_sig.next()[1:].strip())
        exec(fh_sig.next()[1:].strip())
        fh_sig.close()
        # reading sigma
        #data = loadtxt(fh_sig).transpose()
        data = loadtxt(filename).transpose()
        fh_sig.close()
        
        for c in icols[icix]:
            rSigma[2*c-1] += data[2*(c-mincol+1)-1]*(1./noccur[c-1])
            rSigma[2*c]   += data[2*(c-mincol+1)  ]*(1./noccur[c-1])
        
        for c in icols[icix]:
            rs_oo[c-1] += s_oo[c-mincol]*(1./noccur[c-1])
            rEdc[c-1]  += Edc[c-mincol]*(1./noccur[c-1])
        

    # The impurity self-energy has a static contribution
    # equal to s_oo-Edc
    # This self-energy will however go to zero in infinity. No static contribution
    for c in range(len(rSigma)/2): rSigma[2*c+1] -= (rs_oo[c]-rEdc[c])

    
    fout = open(options.osig, 'w')
    print >> fout, '# s_oo=', rs_oo.tolist()
    print >> fout, '# Edc=', rEdc.tolist()
    savetxt(fout,rSigma.transpose())
