#!/usr/bin/env python
import utils,indmffile,sys,re,os,shutil
import optparse, subprocess
from scipy import *

import numpy
if numpy.__version__ == '1.0.1':
    loadtxt = io.read_array
    #
    def savetxt(filename, data):
        io.write_array(filename, data, precision=16)

def union(data):
    " Takes a union of array or list"
    c = []
    for d in data:
        if d not in c:
            c.append(d)
    return c


    
def default_realaxis_mesh():
    # Probably should allow user to control the mesh generation
    # to some extent view program options...
    
    # use gaumesh.py to generate standard mesh
    gaumesh = os.path.join( utils.DmftEnvironment().ROOT, 'gaumesh.py' )
    args = [
        'x0=[0,0]',
        'dx0=[0.25,0.01]',
        'fwhm=[30, 3]',
        'xmin=-20',
        'xmax=20'
        ]
    stdoutdata, stderrdata = subprocess.Popen([gaumesh] + args, stdout=subprocess.PIPE).communicate()
    return [float(x) for x in stdoutdata.split('\n') if x.strip() and not x.strip().startswith('#')]


if __name__=='__main__':
    """ Create a zero input self-energy for dmft0, dmft1 and dmft0
    """
    usage = """usage: %prog [ options ]

    Create a zero input self-energy for dmft0, dmft1 and dmft0
    """

    parser = optparse.OptionParser(usage)
    parser.add_option("-e", "--Edc",  dest="Edc",    type="float", default=0.0, help="Starting double counting and Hartree value of the self-energy. Should be close to U(n-1/2)")
    parser.add_option("-i", "--sinp", dest="insig",  default='sig.inp', help="the mesh will be used from this file.", metavar="FILE")
    parser.add_option("-o", "--sout", dest="outsig", default='sig.inp', help="the result will be written to this file")
    parser.add_option("-n", "--nom",  dest="nom",    type="int", default=500, help="Number of frequency points")
    parser.add_option("-T", "--Temp",  dest="T",     type="float", default=0.0, help="Temperature")
    parser.add_option("-L", "--range", dest="L",     type="float", default=20., help="energy range on real axis")
    # Next, parse the arguments
    (options, args) = parser.parse_args()

    env = utils.W2kEnvironment()
    case = env.case

    print 'Edc=%s case=%s, insig=%s, outsig=%s, nom=%s T=%s' %  (options.Edc, case, options.insig, options.outsig, options.nom, options.T)

    inl = indmffile.Indmfl(case)
    inl.read()
    m_extn = 'dn' if os.path.exists(case+'.indmfl'+'dn') else ''
    if m_extn:
        print 'INFO: case.indmfldn present => magnetic calculation with two dmft2 steps'
        inldn = indmffile.Indmfl(case, 'indmfl'+m_extn)
        inldn.read()

    if options.T>0 and inl.matsubara:
        print '..... creating new matsubara mesh of size '+str(options.nom)+' of T='+str(options.T)
        omega = (arange(1,options.nom,1)*2-1)*pi*options.T
    elif os.path.isfile(options.insig) and os.path.getsize(options.insig)>0:
        # Read the input file
        sigdata = loadtxt(options.insig)  # self-energy from 'sig.inp' on long mesh
        if len(shape(sigdata))==1:
            omega = sigdata
        else:
            omega = (sigdata.transpose())[0]
    else:
        if not inl.matsubara:
            print '..... Could not find '+options.insig+'. Generating default real axis mesh.'
            # omega = arange(-options.L, options.L*(1+2./options.nom), 2*options.L/options.nom)
            omega = default_realaxis_mesh()
        else:
            print '..... Could not find '+options.insig+'. Do not know the temperature. Can not create self-energy!'
            print '..... Boiling out.....'
            sys.exit(1)
        
    print '..... Going over all correlated blocks'
    cols=[]
    for icix in inl.siginds.keys():    # over all imp.problems, even those that are equivalent
        Sigind = inl.siginds[icix]
        cols = sort(union(array(Sigind).flatten().tolist()+cols)).tolist()
    if m_extn:
        for icix in inldn.siginds.keys():    # over all imp.problems, even those that are equivalent
            Sigind = inldn.siginds[icix]
            cols = sort(union(array(Sigind).flatten().tolist()+cols)).tolist()
    if 0 in cols: cols.remove(0)

    print 'cols=', cols
    Nc = cols[-1]

    # saving the original self-energy if necessary
    if options.insig==options.outsig and os.path.isfile(options.insig) and os.path.getsize(options.insig)>0:
        shutil.copy2(options.insig, options.insig+'.bak')


    print 'om=', omega
    
    # writing to the file
    fo = open(options.outsig, 'w')
    print >> fo, '# s_oo=', (ones(Nc)*options.Edc).tolist()
    print >> fo, '#  Edc=', (ones(Nc)*options.Edc).tolist()
    for iom,om in enumerate(omega):
        print >> fo, ("%20.15f "%om), ("0.0 "*(2*Nc))
        
    print options.outsig, 'written to the disc.'
